/**
 * Copyright (c) 2007-2012 EBMWebsourcing, 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl;

import java.io.IOException;
import java.util.Hashtable;
import java.util.Set;

import javax.jbi.messaging.MessageExchange;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.ow2.petals.jmx.api.api.AdminServiceClient;
import org.ow2.petals.jmx.api.api.exception.AdminDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.AdminServiceErrorException;
import org.ow2.petals.jmx.api.impl.exception.PerformActionErrorException;

/**
 * Client API of the JMX Admin service.
 * @author Roland Naudin - EBM WebSourcing
 */
public class AdminServiceClientImpl extends AbstractServiceClient implements AdminServiceClient {

    /**
     * Action : get System Info
     */
    private static final String GET_SYSTEM_INFO = "getSystemInfo";

    /**
     * Action : get Binding Components
     */
    private static final String GET_BINDING_COMPONENTS = "getBindingComponents";

    /**
     * Action : get Services Engines
     */
    private static final String GET_ENGINE_COMPONENTS = "getEngineComponents";

    /**
     * Action : get Component By Name
     */
    private static final String GET_COMPONENT_BY_NAME = "getComponentByName";

    /**
     * Action : check if it is a binding component
     */
    private static final String IS_BINDING = "isBinding";

    /**
     * Action : check if it is a service engine
     */
    private static final String IS_ENGINE = "isEngine";

    /**
     * Action : isExchangeWithConsumerOkayForComponent
     */
    private static final String IS_OK_WITH_CONS = "isExchangeWithConsumerOkayForComponent";

    /**
     * Action : isExchangeWithProviderOkayForComponent
     */
    private static final String IS_OK_WITH_PROV = "isExchangeWithProviderOkayForComponent";

    /**
     * Creates a new instance of {@link AdminServiceClient}
     * 
     * @param petalsDomain
     * @param beanServerConnection
     * @throws AdminDoesNotExistException
     * @throws AdminServiceErrorException
     */
    public AdminServiceClientImpl(final String petalsDomain, final MBeanServerConnection beanServerConnection)
            throws AdminDoesNotExistException, AdminServiceErrorException {
        super(petalsDomain, beanServerConnection);

        try {
            final Hashtable<String, String> attributes = new Hashtable<String, String>();
            attributes.put(JMX_FIELD_NAME, AdminServiceClient.ADMIN_SERVICE_MBEAN_NAME);
            attributes.put(JMX_FIELD_TYPE, JMX_SERVICE_TYPE);
            final ObjectName objName = new ObjectName(petalsDomain, attributes);
            final Set<ObjectName> objNames = this.mBeanServerConnection.queryNames(objName, null);
            if (objNames != null && objNames.size() == 1) {
                this.mbeanName = objNames.iterator().next();
            } else {
                throw new AdminDoesNotExistException();
            }
        } catch (final MalformedObjectNameException e) {
            throw new AdminServiceErrorException(e);
        } catch (final IOException e) {
            throw new AdminServiceErrorException(e);
        }
    }

    @Override
    public String getSystemInfo() throws AdminServiceErrorException {

        try {
            return (String) this.performAction(GET_SYSTEM_INFO, null, null);
        } catch (final PerformActionErrorException e) {
            throw new AdminServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new AdminServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public ObjectName[] getBindingComponents() throws AdminServiceErrorException {

        try {
            return (ObjectName[]) this.performAction(GET_BINDING_COMPONENTS, null, null);
        } catch (final PerformActionErrorException e) {
            throw new AdminServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new AdminServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public ObjectName[] getEngineComponents() throws AdminServiceErrorException {

        try {
            return (ObjectName[]) this.performAction(GET_ENGINE_COMPONENTS, null, null);
        } catch (final PerformActionErrorException e) {
            throw new AdminServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new AdminServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public ObjectName getComponentByName(final String component) throws AdminServiceErrorException {

        final Object[] objectsLoad = new Object[] { component };
        final String[] stringsLoad = new String[] { String.class.getName() };
        try {
            return (ObjectName) this.performAction(GET_COMPONENT_BY_NAME, objectsLoad, stringsLoad);
        } catch (final PerformActionErrorException e) {
            throw new AdminServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new AdminServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public boolean isBinding(final String componentName) throws AdminServiceErrorException {

        final Object[] objectsLoad = new Object[] { componentName };
        final String[] stringsLoad = new String[] { String.class.getName() };
        try {
            return ((Boolean) this.performAction(IS_BINDING, objectsLoad, stringsLoad)).booleanValue();
        } catch (final PerformActionErrorException e) {
            throw new AdminServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new AdminServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public boolean isEngine(final String componentName) throws AdminServiceErrorException {

        final Object[] objectsLoad = new Object[] { componentName };
        final String[] stringsLoad = new String[] { String.class.getName() };
        try {
            return ((Boolean) this.performAction(IS_ENGINE, objectsLoad, stringsLoad)).booleanValue();
        } catch (final PerformActionErrorException e) {
            throw new AdminServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new AdminServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public boolean isExchangeWithConsumerOkayForComponent(final String componentName,
            final ServiceEndpoint providerEndpoint, final MessageExchange exchange)
            throws AdminServiceErrorException {

        final Object[] objectsLoad = new Object[] { componentName, providerEndpoint, exchange };
        final String[] stringsLoad = new String[] { String.class.getName(),
                ServiceEndpoint.class.getName(), MessageExchange.class.getName() };
        try {
            return ((Boolean) this.performAction(IS_OK_WITH_CONS, objectsLoad, stringsLoad))
                    .booleanValue();
        } catch (final PerformActionErrorException e) {
            throw new AdminServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new AdminServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public boolean isExchangeWithProviderOkayForComponent(final String componentName,
            final ServiceEndpoint providerEndpoint, final MessageExchange exchange)
            throws AdminServiceErrorException {

        final Object[] objectsLoad = new Object[] { componentName, providerEndpoint, exchange };
        final String[] stringsLoad = new String[] { String.class.getName(),
                ServiceEndpoint.class.getName(), MessageExchange.class.getName() };
        try {
            return ((Boolean) this.performAction(IS_OK_WITH_PROV, objectsLoad, stringsLoad))
                    .booleanValue();
        } catch (final PerformActionErrorException e) {
            throw new AdminServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new AdminServiceErrorException("Unexpected result type", e);
        }
    }
}
