/**
 * Copyright (c) 2007-2010 Capgemini, 2010-2012 EBMWebsourcing, 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl;

import java.io.IOException;
import java.net.URL;
import java.util.Hashtable;
import java.util.Set;

import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.ow2.petals.jmx.api.api.DeploymentServiceClient;
import org.ow2.petals.jmx.api.api.exception.DeploymentServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.DeploymentServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.ServiceAssemblyDoesNotExistException;
import org.ow2.petals.jmx.api.impl.exception.PerformActionErrorException;
import org.ow2.petals.jmx.api.impl.exception.PerformActionRemoteErrorException;

/**
 * Client API of the JMX deployment service.
 * @author Christophe Deneux - Capgemini Sud
 */
public final class DeploymentServiceClientImpl extends AbstractServiceClient implements
        DeploymentServiceClient {

    /**
     * Action : deploy
     */
    private static final String DEPLOY = "deploy";

    /**
     * Action : force undeploy
     */
    private static final String FORCE_UNDEPLOY = "forceUndeploy";

    /**
     * Action : getDeployedServiceAssemblies
     */
    private static final String GET_DEPLOYED_SAS = "getDeployedServiceAssemblies";

    /**
     * Action : getDeployedServiceAssembliesForComponent
     */
    private static final String GET_DEPLOYED_SAS_FOR_COMPONENT = "getDeployedServiceAssembliesForComponent";

    /**
     * Action : getServiceAssemblyDescriptor
     */
    private static final String GET_SERVICE_ASSEMBLY_DESCRIPTOR = "getServiceAssemblyDescriptor";

    /**
     * Action : getServiceUnitForServiceAssembly
     */
    private static final String GET_SERVICE_UNIT_FOR_SA = "getServiceUnitForServiceAssembly";

    /**
     * Action : getState
     */
    private static final String GET_STATE = "getState";

    /**
     * Action : shutdown
     */
    private static final String SHUTDOWN = "shutDown";

    /**
     * Action : shutdownAllServiceAsssemblies
     */
    private static final String SHUTDOWN_ALL_SAS = "shutdownAllServiceAssemblies";

    /**
     * Action : start
     */
    private static final String START = "start";

    /**
     * Action : startAllServiceAsssemblies
     */
    private static final String START_ALL_SAS = "startAllServiceAssemblies";

    /**
     * Action : stop
     */
    private static final String STOP = "stop";

    /**
     * Action : stopAllServiceAsssemblies
     */
    private static final String STOP_ALL_SAS = "stopAllServiceAssemblies";

    /**
     * Action : undeploy
     */
    private static final String UNDEPLOY = "undeploy";

    /**
     * Action : undeployAllServiceAssemblies
     */
    private static final String UNDEPLOY_ALL_SAS = "undeployAllServiceAssemblies";

    /**
     * Creates a new instance of {@link DeploymentServiceClient}
     * 
     * @param petalsDomain
     * @param mBeanServerConnection
     * @throws DeploymentServiceDoesNotExistException
     * @throws DeploymentServiceErrorException
     */
    public DeploymentServiceClientImpl(final String petalsDomain,
            MBeanServerConnection mBeanServerConnection)
            throws DeploymentServiceDoesNotExistException, DeploymentServiceErrorException {

        super(petalsDomain, mBeanServerConnection);

        try {
            final Hashtable<String, String> attributes = new Hashtable<String, String>();
            attributes.put(JMX_FIELD_NAME, DeploymentServiceClient.DEPLOYMENT_SERVICE_MBEAN_NAME);
            attributes.put(JMX_FIELD_TYPE, JMX_SERVICE_TYPE);
            final ObjectName objName = new ObjectName(petalsDomain, attributes);
            final Set<ObjectName> objNames = this.mBeanServerConnection.queryNames(objName, null);
            if (objNames != null && objNames.size() == 1) {
                this.mbeanName = objNames.iterator().next();
            } else {
                throw new DeploymentServiceDoesNotExistException();
            }
        } catch (final MalformedObjectNameException e) {
            throw new DeploymentServiceErrorException(e);
        } catch (final IOException e) {
            throw new DeploymentServiceErrorException(e);
        }
    }

    @Override
    public String deploy(final URL packageURL) throws DeploymentServiceErrorException {

        final Object[] objects = new Object[] { packageURL.toString() };
        final String[] strings = new String[] { String.class.getName() };

        try {
            return (String) this.performAction(DEPLOY, objects, strings);
        } catch (final PerformActionErrorException e) {
            throw new DeploymentServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new DeploymentServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public boolean forceUndeploy(final String serviceAssemblyName)
            throws DeploymentServiceErrorException {

        final Object[] objects = new Object[] { serviceAssemblyName };
        final String[] strings = new String[] { String.class.getName() };

        try {
            return (Boolean) this.performAction(FORCE_UNDEPLOY, objects, strings);
        } catch (final PerformActionErrorException e) {
            throw new DeploymentServiceErrorException(e);
        } catch (final ClassCastException | NullPointerException e) {
            throw new DeploymentServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public String[] getDeployedServiceAssemblies() throws DeploymentServiceErrorException {

        try {
            return (String[]) this.performAction(GET_DEPLOYED_SAS, null, null);
        } catch (final PerformActionErrorException e) {
            throw new DeploymentServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new DeploymentServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public String[] getDeployedServiceAssembliesForComponent(final String componentName)
            throws DeploymentServiceErrorException {

        final Object[] objects = new Object[] { componentName };
        final String[] strings = new String[] { String.class.getName() };

        try {
            return (String[]) this.performAction(GET_DEPLOYED_SAS_FOR_COMPONENT, objects, strings);
        } catch (final PerformActionErrorException e) {
            throw new DeploymentServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new DeploymentServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public String getServiceAssemblyDescriptor(final String serviceAssemblyName)
            throws DeploymentServiceErrorException {

        final Object[] objects = new Object[] { serviceAssemblyName };
        final String[] strings = new String[] { String.class.getName() };

        try {
            return (String) this.performAction(GET_SERVICE_ASSEMBLY_DESCRIPTOR, objects, strings);
        } catch (final PerformActionErrorException e) {
            throw new DeploymentServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new DeploymentServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public String[] getServiceUnitForServiceAssembly(final String serviceAssemblyName)
            throws DeploymentServiceErrorException {

        final Object[] objects = new Object[] { serviceAssemblyName };
        final String[] strings = new String[] { String.class.getName() };

        try {
            return (String[]) this.performAction(GET_SERVICE_UNIT_FOR_SA, objects, strings);
        } catch (final PerformActionErrorException e) {
            throw new DeploymentServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new DeploymentServiceErrorException("Unexpected result type", e);
        }

    }

    @Override
    public State getState(final String serviceAssemblyName)
            throws ServiceAssemblyDoesNotExistException, DeploymentServiceErrorException {

        final Object[] objects = new Object[] { serviceAssemblyName };
        final String[] strings = new String[] { String.class.getName() };

        try {
            final String state = (String) this.performAction(GET_STATE, objects, strings);
            return State.parse(state);
        } catch (final PerformActionRemoteErrorException e) {
            // The service assembly is not deployed
            throw new ServiceAssemblyDoesNotExistException(serviceAssemblyName);
        } catch (final PerformActionErrorException e) {
            throw new DeploymentServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new DeploymentServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public boolean isServiceAssemblyDeployed(final String serviceAssembly)
            throws DeploymentServiceErrorException {
        boolean result = false;

        final String[] deployedSAs = this.getDeployedServiceAssemblies();
        for (final String deployedSA : deployedSAs) {
            if (deployedSA.equals(serviceAssembly)) {
                result = true;
                break;
            }
        }

        return result;
    }

    @Override
    public String shutdown(final String serviceAssemblyName) throws DeploymentServiceErrorException {

        final Object[] objects = new Object[] { serviceAssemblyName };
        final String[] strings = new String[] { String.class.getName() };

        try {
            return (String) this.performAction(SHUTDOWN, objects, strings);
        } catch (final PerformActionErrorException e) {
            throw new DeploymentServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new DeploymentServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public String[] shutdownAllServiceAssemblies() throws DeploymentServiceErrorException {

        try {
            return (String[]) this.performAction(SHUTDOWN_ALL_SAS, null, null);
        } catch (final PerformActionErrorException e) {
            throw new DeploymentServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new DeploymentServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public String start(final String serviceAssemblyName) throws DeploymentServiceErrorException {

        final Object[] objects = new Object[] { serviceAssemblyName };
        final String[] strings = new String[] { String.class.getName() };

        try {
            return (String) this.performAction(START, objects, strings);
        } catch (final PerformActionErrorException e) {
            throw new DeploymentServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new DeploymentServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public String[] startAllServiceAssemblies() throws DeploymentServiceErrorException {

        try {
            return (String[]) this.performAction(START_ALL_SAS, null, null);
        } catch (final PerformActionErrorException e) {
            throw new DeploymentServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new DeploymentServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public String stop(final String serviceAssemblyName) throws DeploymentServiceErrorException {

        final Object[] objects = new Object[] { serviceAssemblyName };
        final String[] strings = new String[] { String.class.getName() };

        try {
            return (String) this.performAction(STOP, objects, strings);
        } catch (final PerformActionErrorException e) {
            throw new DeploymentServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new DeploymentServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public String[] stopAllServiceAssemblies() throws DeploymentServiceErrorException {

        try {
            return (String[]) this.performAction(STOP_ALL_SAS, null, null);
        } catch (final PerformActionErrorException e) {
            throw new DeploymentServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new DeploymentServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public String undeploy(final String serviceAssemblyName) throws DeploymentServiceErrorException {

        final Object[] objects = new Object[] { serviceAssemblyName };
        final String[] strings = new String[] { String.class.getName() };

        try {
            return (String) this.performAction(UNDEPLOY, objects, strings);
        } catch (final PerformActionErrorException e) {
            throw new DeploymentServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new DeploymentServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public String[] undeployAllServiceAssemblies(final boolean force)
            throws DeploymentServiceErrorException {

        final Object[] objects = new Object[] { force };
        final String[] strings = new String[] { boolean.class.getName() };

        try {
            return (String[]) this.performAction(UNDEPLOY_ALL_SAS, objects, strings);
        } catch (final PerformActionErrorException e) {
            throw new DeploymentServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new DeploymentServiceErrorException("Unexpected result type", e);
        }
    }

}
