/**
 * Copyright (c) 2008-2012 EBMWebsourcing, 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl;

import java.io.IOException;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.ow2.petals.clientserverapi.jbi.messaging.endpoint.exception.EndpointDirectoryException;
import org.ow2.petals.jmx.api.api.EndpointDirectoryClient;
import org.ow2.petals.jmx.api.api.exception.AdminDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.AdminServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.EndpointDirectoryServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.EndpointDirectoryServiceErrorException;
import org.ow2.petals.jmx.api.impl.exception.PerformActionErrorException;

/**
 * Client API of the JMX Endpoint directory service.
 * 
 * @author Maxime Carpentier - EBM WebSourcing
 */
public class EndpointDirectoryClientImpl extends AbstractServiceClient implements
        EndpointDirectoryClient {

    /**
     * Action : get all internal endpoints
     */
    protected static final String GET_ALL_INTERNAL_ENDPOINTS = "getAllInternalEndpoints";

    /**
     * Action : get all external endpoints
     */
    protected static final String GET_ALL_EXTERNAL_ENDPOINTS = "getAllExternalEndpoints";

    /**
     * Action : get all endpoints
     */
    protected static final String GET_ALL_ENDPOINTS = "getAllEndpoints";

    /**
     * Action : get the description of a given endpoint and service
     */
    protected static final String GET_ENDPOINT_DESCRIPTION = "getDescription";

    /**
     * Creates a new instance of {@link EndpointDirectoryClient}
     * 
     * @param petalsDomain
     * @param beanServerConnection
     * @throws AdminDoesNotExistException
     * @throws AdminServiceErrorException
     */
    public EndpointDirectoryClientImpl(final String petalsDomain,
            final MBeanServerConnection beanServerConnection)
            throws EndpointDirectoryServiceDoesNotExistException,
            EndpointDirectoryServiceErrorException {
        super(petalsDomain, beanServerConnection);

        try {
            final Hashtable<String, String> attributes = new Hashtable<String, String>();
            attributes.put(JMX_FIELD_NAME, EndpointDirectoryClient.ENDPOINT_DIRECTORY_MBEAN_NAME);
            attributes.put(JMX_FIELD_TYPE, JMX_SERVICE_TYPE);
            final ObjectName objName = new ObjectName(petalsDomain, attributes);
            final Set<ObjectName> objNames = this.mBeanServerConnection.queryNames(objName, null);
            if (objNames != null && objNames.size() == 1) {
                this.mbeanName = objNames.iterator().next();
            } else {
                throw new EndpointDirectoryServiceDoesNotExistException();
            }
        } catch (final MalformedObjectNameException e) {
            throw new EndpointDirectoryServiceErrorException(e);
        } catch (final IOException e) {
            throw new EndpointDirectoryServiceErrorException(e);
        }
    }

    /**
     * {@inheritDoc}
     * 
     * <p>
     * If an error occurs performing the JMX operation, the cause of the {@link EndpointDirectoryException} will be
     * {@link EndpointDirectoryServiceErrorException}.
     * </p>
     */
    @Override
    public List<Map<String, Object>> getAllInternalEndpoints() throws EndpointDirectoryException {

        try {
            try {
                return (List<Map<String, Object>>) this.performAction(GET_ALL_INTERNAL_ENDPOINTS, null, null);
            } catch (final PerformActionErrorException e) {
                throw new EndpointDirectoryServiceErrorException(e);
            } catch (final ClassCastException e) {
                throw new EndpointDirectoryServiceErrorException("Unexpected result type", e);
            }
        } catch (final EndpointDirectoryServiceErrorException e) {
            throw new EndpointDirectoryException(e);
        }
    }

    /**
     * {@inheritDoc}
     * 
     * <p>
     * If an error occurs performing the JMX operation, the cause of the {@link EndpointDirectoryException} will be
     * {@link EndpointDirectoryServiceErrorException}.
     * </p>
     */
    @Override
    public List<Map<String, Object>> getAllExternalEndpoints() throws EndpointDirectoryException {

        try {
            try {
                return (List<Map<String, Object>>) this.performAction(GET_ALL_EXTERNAL_ENDPOINTS, null, null);
            } catch (final PerformActionErrorException e) {
                throw new EndpointDirectoryServiceErrorException(e);
            } catch (final ClassCastException e) {
                throw new EndpointDirectoryServiceErrorException("Unexpected result type", e);
            }
        } catch (final EndpointDirectoryServiceErrorException e) {
            throw new EndpointDirectoryException(e);
        }
    }

    /**
     * {@inheritDoc}
     * 
     * <p>
     * If an error occurs performing the JMX operation, the cause of the {@link EndpointDirectoryException} will be
     * {@link EndpointDirectoryServiceErrorException}.
     * </p>
     */
    @Override
    public List<Map<String, Object>> getAllEndpoints() throws EndpointDirectoryException {

        try {
            try {
                return (List<Map<String, Object>>) this.performAction(GET_ALL_ENDPOINTS, null, null);
            } catch (final PerformActionErrorException e) {
                throw new EndpointDirectoryServiceErrorException(e);
            } catch (final ClassCastException e) {
                throw new EndpointDirectoryServiceErrorException("Unexpected result type", e);
            }
        } catch (final EndpointDirectoryServiceErrorException e) {
            throw new EndpointDirectoryException(e);
        }
    }

    /**
     * {@inheritDoc}
     * 
     * <p>
     * If an error occurs performing the JMX operation, the cause of the {@link EndpointDirectoryException} will be
     * {@link EndpointDirectoryServiceErrorException}.
     * </p>
     */
    @Override
    public String getDescription(final String serviceName, final String endpointName) throws EndpointDirectoryException {

        final Object[] objects = new Object[] { serviceName, endpointName };
        final String[] strings = new String[] { String.class.getName(), String.class.getName() };

        try {
            try {
                return (String) this.performAction(GET_ENDPOINT_DESCRIPTION, objects, strings);
            } catch (final PerformActionErrorException e) {
                throw new EndpointDirectoryServiceErrorException(e);
            } catch (final ClassCastException e) {
                throw new EndpointDirectoryServiceErrorException("Unexpected result type", e);
            }
        } catch (final EndpointDirectoryServiceErrorException e) {
            throw new EndpointDirectoryException(e);
        }
    }
}
