/**
 * Copyright (c) 2007-2010 Capgemini, 2010-2012 EBMWebsourcing, 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl;

import java.io.IOException;
import java.net.URL;
import java.util.Hashtable;
import java.util.Set;

import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.ow2.petals.jmx.api.api.InstallationServiceClient;
import org.ow2.petals.jmx.api.api.InstallerComponentClient;
import org.ow2.petals.jmx.api.api.exception.InstallationServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.InstallationServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.InstallerComponentDoesNotExistException;
import org.ow2.petals.jmx.api.impl.exception.PerformActionErrorException;

/**
 * Client API of the JMX installation service.
 * @author Christophe Deneux - Capgemini Sud
 */
public final class InstallationServiceClientImpl extends AbstractServiceClient implements
        InstallationServiceClient {

    /**
     * Action : forceUnloadInstaller
     */
    private static final String FORCE_UNLOAD_INSTALLER = "forceUnloadInstaller";

    /**
     * Action : getInstalledComponentsForSharedLibrary
     */
    private static final String GET_COMPONENTS_FOR_SL = "getInstalledComponentsForSharedLibrary";

    /**
     * Action : getInstalledSharedLibraries
     */
    private static final String GET_INSTALLED_SLS = "getInstalledSharedLibraries";

    /**
     * Action : getInstallers
     */
    private static final String GET_INSTALLERS = "getInstallers";

    /**
     * Action : installSharedLibrary
     */
    private static final String INSTALL_SL = "installSharedLibrary";

    /**
     * Action : loadInstaller
     */
    private static final String LOAD_INSTALLER = "loadInstaller";

    /**
     * Action : loadNewInstaller
     */
    private static final String LOAD_NEW_INSTALLER = "loadNewInstaller";

    /**
     * Action : shutdownComponent
     */
    private static final String SHUTDOWN_ALL_COMPONENTS = "shutdownAllComponents";

    /**
     * Action : shutdownComponent
     */
    private static final String SHUTDOWN_COMPONENT = "shutdownComponent";

    /**
     * Action : startAllComponent
     */
    private static final String START_ALL_COMPONENTS = "startAllComponents";

    /**
     * Action : startComponent
     */
    private static final String START_COMPONENT = "startComponent";

    /**
     * Action : stopAllComponent
     */
    private static final String STOP_ALL_COMPONENTS = "stopAllComponents";

    /**
     * Action : stopComponent
     */
    private static final String STOP_COMPONENT = "stopComponent";

    /**
     * Action : uninstallAllComponents
     */
    private static final String UNINSTALL_ALL_COMPONENTS = "uninstallAllComponents";

    /**
     * Action : uninstallAllSharedLibrary
     */
    private static final String UNINSTALL_ALL_SL = "uninstallAllSharedLibraries";

    /**
     * Action : uninstallSharedLibrary
     */
    private static final String UNINSTALL_SL = "uninstallSharedLibrary";

    /**
     * Action : unloadAllInstallers.
     */
    private static final String UNLOAD_ALL_INSTALLERS = "unloadAllInstallers";

    /**
     * Action : loadNewInstaller
     */
    private static final String UNLOAD_INSTALLER = "unloadInstaller";

    /**
     * Creates a new instance of {@link InstallationServiceClient}
     * 
     * @param petalsDomain
     * @param mBeanServerConnection
     * @throws InstallationServiceDoesNotExistException
     * @throws InstallationServiceErrorException
     */
    public InstallationServiceClientImpl(final String petalsDomain, final MBeanServerConnection mBeanServerConnection)
            throws InstallationServiceDoesNotExistException, InstallationServiceErrorException {

        super(petalsDomain, mBeanServerConnection);

        try {
            final Hashtable<String, String> attributes = new Hashtable<String, String>();
            attributes.put(JMX_FIELD_NAME,
                    InstallationServiceClient.INSTALLATION_SERVICE_MBEAN_NAME);
            attributes.put(JMX_FIELD_TYPE, JMX_SERVICE_TYPE);
            final ObjectName objName = new ObjectName(petalsDomain, attributes);
            final Set<ObjectName> objNames = this.mBeanServerConnection.queryNames(objName, null);
            if (objNames != null && objNames.size() == 1) {
                this.mbeanName = objNames.iterator().next();
            } else {
                throw new InstallationServiceDoesNotExistException(objName.toString());
            }
        } catch (final MalformedObjectNameException e) {
            throw new InstallationServiceErrorException(e);
        } catch (final IOException e) {
            throw new InstallationServiceErrorException(e);
        }
    }

    @Override
    public boolean forceUnloadInstaller(final String componentName)
            throws InstallationServiceErrorException {

        final Object[] objects = new Object[] { componentName };
        final String[] strings = new String[] { String.class.getName() };

        try {
            return ((Boolean) this.performAction(FORCE_UNLOAD_INSTALLER, objects, strings)).booleanValue();
        } catch (final PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public String[] getInstalledComponentsForSharedlibrary(final String sharedLibraryName,
            final String sharedLibraryVersion)
            throws InstallationServiceErrorException {

        final Object[] objects = new Object[] { sharedLibraryName, sharedLibraryVersion };
        final String[] strings = new String[] { String.class.getName(), String.class.getName() };

        try {
            return (String[]) this.performAction(GET_COMPONENTS_FOR_SL, objects, strings);
        } catch (final PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public String[][] getInstalledSharedLibraries() throws InstallationServiceErrorException {

        try {
            return (String[][]) this.performAction(GET_INSTALLED_SLS, null, null);
        } catch (final PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public String[] getInstallers() throws InstallationServiceErrorException {

        try {
            return (String[]) this.performAction(GET_INSTALLERS, null, null);
        } catch (final PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public String installSharedLibrary(final URL sharedLibraryURL)
            throws InstallationServiceErrorException {

        final Object[] objects = new Object[] { sharedLibraryURL.toString() };
        final String[] strings = new String[] { String.class.getName() };

        try {
            return (String) this.performAction(INSTALL_SL, objects, strings);
        } catch (final PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public InstallerComponentClient loadInstaller(final String componentName)
            throws InstallerComponentDoesNotExistException, InstallationServiceErrorException {

        final Object[] objects = new Object[] { componentName };
        final String[] strings = new String[] { String.class.getName() };

        try {
            final ObjectName installerObjName = (ObjectName) this.performAction(LOAD_INSTALLER, objects, strings);

            if (installerObjName == null) {
                throw new InstallerComponentDoesNotExistException(componentName);
            } else {
                return new InstallerComponentClientImpl(this.petalsDomain, installerObjName, this.mBeanServerConnection);
            }
        } catch (final PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public InstallerComponentClient loadNewInstaller(final URL packageURL)
            throws InstallationServiceErrorException {

        final Object[] objects = new Object[] { packageURL.toString() };
        final String[] strings = new String[] { String.class.getName() };

        try {
            final ObjectName installerObjName = (ObjectName) this.performAction(LOAD_NEW_INSTALLER, objects, strings);
            return new InstallerComponentClientImpl(this.petalsDomain, installerObjName, this.mBeanServerConnection);
        } catch (final PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public String[] shutdownAllComponents() throws InstallationServiceErrorException {

        try {
            return (String[]) this.performAction(SHUTDOWN_ALL_COMPONENTS, null, null);
        } catch (final PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public boolean shutdownComponent(String componentName) throws InstallationServiceErrorException {

        final Object[] objects = new Object[] { componentName };
        final String[] strings = new String[] { String.class.getName() };

        try {
            return ((Boolean) this.performAction(SHUTDOWN_COMPONENT, objects, strings)).booleanValue();
        } catch (final PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public String[] startAllComponents() throws InstallationServiceErrorException {

        try {
            return (String[]) this.performAction(START_ALL_COMPONENTS, null, null);
        } catch (final PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public boolean startComponent(final String componentName) throws InstallationServiceErrorException {

        final Object[] objects = new Object[] { componentName };
        final String[] strings = new String[] { String.class.getName() };

        try {
            return ((Boolean) this.performAction(START_COMPONENT, objects, strings)).booleanValue();
        } catch (final PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public String[] stopAllComponents() throws InstallationServiceErrorException {

        try {
            return (String[]) this.performAction(STOP_ALL_COMPONENTS, null, null);
        } catch (final PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public boolean stopComponent(String componentName) throws InstallationServiceErrorException {

        final Object[] objects = new Object[] { componentName };
        final String[] strings = new String[] { String.class.getName() };

        try {
            return ((Boolean) this.performAction(STOP_COMPONENT, objects, strings)).booleanValue();
        } catch (final PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type");
        }
    }

    @Override
    public String[] uninstallAllComponents() throws InstallationServiceErrorException {

        try {
            return (String[]) this.performAction(UNINSTALL_ALL_COMPONENTS, null, null);
        } catch (final PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type");
        }
    }

    @Override
    public String[] uninstallAllSharedLibrary() throws InstallationServiceErrorException {

        try {
            return (String[]) this.performAction(UNINSTALL_ALL_SL, null, null);
        } catch (final PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public boolean uninstallSharedLibrary(final String sharedLibraryName, final String sharedLibraryVersion)
            throws InstallationServiceErrorException {

        final Object[] objects = new Object[] { sharedLibraryName, sharedLibraryVersion };
        final String[] strings = new String[] { String.class.getName(), String.class.getName() };

        try {
            return ((Boolean) this.performAction(UNINSTALL_SL, objects, strings)).booleanValue();
        } catch (final PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public String[] unloadAllInstallers(final boolean force) throws InstallationServiceErrorException {

        final Object[] objects = new Object[] { force };
        final String[] strings = new String[] { boolean.class.getName() };

        try {
            return (String[]) this.performAction(UNLOAD_ALL_INSTALLERS, objects, strings);
        } catch (final PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public boolean unloadInstaller(final String componentName)
            throws InstallationServiceErrorException {

        final Object[] objects = new Object[] { componentName, true };
        final String[] strings = new String[] { String.class.getName(), boolean.class.getName() };

        try {
            return ((Boolean) this.performAction(UNLOAD_INSTALLER, objects, strings)).booleanValue();
        } catch (final PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type", e);
        }
    }
}
