/**
 * Copyright (c) 2007-2010 Capgemini, 2010-2012 EBMWebsourcing, 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl;

import java.io.IOException;
import java.net.MalformedURLException;
import java.util.HashMap;
import java.util.Map;

import javax.management.MBeanServer;
import javax.management.MBeanServerConnection;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;

import org.ow2.petals.jmx.api.api.JMXClient;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;

/**
 * A Petals JMX Server Connection.
 * @author Christophe Deneux - Capgemini Sud
 * @author Cedric Girodengo - Capgemini Sud
 */
public final class JMXConnection {

    private JMXConnector connector;

    private MBeanServerConnection mBeanServerConnection;
    
    /**
     * Creates a new instance of {@link JMXConnection} for a remote JMX Server
     * 
     * @param connector
     * @throws ConnectionErrorException
     */
    public JMXConnection(final JMXConnector connector) throws ConnectionErrorException {

        this.init(connector);
    }

    private void init(final JMXConnector connector) throws ConnectionErrorException {

        if (connector == null) {
            throw new ConnectionErrorException("Can not connect using a null JMXConnector");
        }

        this.connector = connector;
        try {
            this.mBeanServerConnection = connector.getMBeanServerConnection();
        } catch (final IOException e) {
            throw new ConnectionErrorException(e);
        }
    }

    /**
     * Creates a new instance of {@link JMXConnection}
     * 
     * @param host
     * @param port
     *            the JMX port used to get the JMX RMI stub.<br>
     *            If null, dynamic mode is used (may not work with proxies).
     * @param username
     * @param password
     * @throws ConnectionErrorException
     */
    public JMXConnection(final String host, final int port, final String username, final String password)
            throws ConnectionErrorException {

        final Map<String, Object> args = new HashMap<>();

        // set credentials
        if (!"".equals(username)) {
            final String[] credentials;
            if (password == null) {
                credentials = new String[] { username, "" };
            } else {
                credentials = new String[] { username, password };
            }
            args.put(JMXConnector.CREDENTIALS, credentials);
        }

        final String jmxServiceURL = "service:jmx:rmi:///jndi/rmi://" + host + ":" + port + "/"
                + JMXClient.PETALS_JMX_RMI_STUB_NAME;

        try {
            final JMXServiceURL url = new JMXServiceURL(jmxServiceURL);

            this.init(JMXConnectorFactory.connect(url, args));

        } catch (final MalformedURLException e) {
            throw new ConnectionErrorException(e);
        } catch (final IOException e) {
            throw new ConnectionErrorException("Can not connect using URI : "
                    + jmxServiceURL + " with username " + username, e);
        } catch (final SecurityException e) {
            throw new ConnectionErrorException(e);
        }
    }

    /**
     * Creates a new instance of {@link JMXConnection} for an embedded JMX
     * Server.
     * 
     * @param mBeanServer
     *            The embedded JMX server.
     * @throws ConnectionErrorException
     */
    public JMXConnection(final MBeanServer mBeanServer) throws ConnectionErrorException {

        this.connector = null;

        if (mBeanServer == null) {
            throw new ConnectionErrorException("Can not connect using a null MBeanServer");
        }

        this.mBeanServerConnection = mBeanServer;
    }

    /**
     * @return The {@link MBeanServerConnection} of the JMX connection.
     */
    public MBeanServerConnection getMBeanServerConnection() {
        return this.mBeanServerConnection;
    }

    /**
     * Close the JMX connection
     * 
     * @throws ConnectionErrorException
     */
    public void disconnect() throws ConnectionErrorException {

        if (this.connector != null) {
            // Remote connection, we close it
            try {
                this.connector.close();
            } catch (final IOException e) {
                throw new ConnectionErrorException(e);
            }
        }
    }
}
