/**
 * Copyright (c) 2008-2012 EBMWebsourcing, 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl;

import java.io.IOException;
import java.util.Set;

import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.openmbean.CompositeDataSupport;

import org.ow2.petals.jmx.api.api.SystemMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.exception.AttributeErrorException;
import org.ow2.petals.jmx.api.api.exception.SystemMonitoringDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.SystemMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.impl.exception.PerformActionErrorException;

/**
 * Client API of the JMX System Monitoring service.
 * @author Maxime Carpentier - EBM WebSourcing
 */
public class SystemMonitoringServiceClientImpl extends AbstractServiceClient implements
        SystemMonitoringServiceClient {

    /**
     * Attributes for ClassLoading MBean
     */
    protected static final String LOADED_CLASS_COUNT = "LoadedClassCount";

    protected static final String TOTAL_LOADED_CLASS_COUNT = "TotalLoadedClassCount";

    protected static final String UNLOADED_CLASS_COUNT = "UnloadedClassCount";

    /**
     * Attributes for Runtime MBean
     */
    protected static final String UP_TIME = "Uptime";

    /**
     * Attributes for Threading MBean
     */
    protected static final String PEAK_THREAD_COUNT = "PeakThreadCount";

    protected static final String DAEMON_THREAD_COUNT = "DaemonThreadCount";

    protected static final String THREAD_COUNT = "ThreadCount";

    protected static final String TOTAL_STARTED_THREAD_COUNT = "TotalStartedThreadCount";

    /**
     * Attributes for Memory MBean
     */
    protected static final String HEAP_MEMORY_USAGE = "HeapMemoryUsage";

    protected static final String OBJECT_PENDING_FINALIZATION_COUNT = "ObjectPendingFinalizationCount";

    /**
     * Attributes for OS MBean : some attributes are platform specific
     */
    protected static final String AVAILABLE_PROCESSORS = "AvailableProcessors";

    protected static final String PROCESS_CPU_TIME = "ProcessCpuTime";

    protected static final String TOTAL_PHYSICAL_MEMORY_SIZE = "TotalPhysicalMemorySize";

    protected static final String FREE_PHYSICAL_MEMORY_SIZE = "FreePhysicalMemorySize";

    protected static final String COMMITTED_VIRTUAL_MEMORY_SIZE = "CommittedVirtualMemorySize";

    /** MBeans of java.lang domain* */
    private static final String OPERATING_SYSTEM = "OperatingSystem";

    private static final String RUNTIME = "Runtime";

    private static final String THREADING = "Threading";

    private static final String MEMORY = "Memory";

    private static final String CLASS_LOADING = "ClassLoading";

    private ObjectName operatingSystemMBeanName;

    private ObjectName runtimeMBeanName;

    private ObjectName threadingMBeanName;

    private ObjectName memoryMBeanName;

    private ObjectName classLoadingMBeanName;

    /**
     * Creates a new instance of {@link SystemMonitoringServiceClient}
     * Check if all needed beans exists
     * 
     * @param domain
     * @param beanServerConnection
     * @throws SystemMonitoringDoesNotExistException if a needed bean does not exists
     */
    public SystemMonitoringServiceClientImpl(String domain,
            MBeanServerConnection beanServerConnection)
            throws SystemMonitoringDoesNotExistException, SystemMonitoringServiceErrorException {
        super(domain, beanServerConnection);

        try {
            Set<ObjectName> objNames;
            ObjectName objName = new ObjectName(domain + ":*");
            objNames = this.mBeanServerConnection.queryNames(objName, null);

            for (ObjectName bean : objNames) {
                if (bean.getCanonicalName().equalsIgnoreCase(
                        domain + ":" + JMX_FIELD_TYPE + "=" + OPERATING_SYSTEM)) {
                    this.operatingSystemMBeanName = bean;
                } else if (bean.getCanonicalName().equalsIgnoreCase(
                        domain + ":" + JMX_FIELD_TYPE + "=" + RUNTIME)) {
                    this.runtimeMBeanName = bean;
                } else if (bean.getCanonicalName().equalsIgnoreCase(
                        domain + ":" + JMX_FIELD_TYPE + "=" + THREADING)) {
                    this.threadingMBeanName = bean;
                } else if (bean.getCanonicalName().equalsIgnoreCase(
                        domain + ":" + JMX_FIELD_TYPE + "=" + MEMORY)) {
                    this.memoryMBeanName = bean;
                } else if (bean.getCanonicalName().equalsIgnoreCase(
                        domain + ":" + JMX_FIELD_TYPE + "=" + CLASS_LOADING)) {
                    this.classLoadingMBeanName = bean;
                }
            }

            if (this.operatingSystemMBeanName == null) {
                throw new SystemMonitoringDoesNotExistException("Can't find " + OPERATING_SYSTEM
                        + " bean");
            }
            if (this.runtimeMBeanName == null) {
                throw new SystemMonitoringDoesNotExistException("Can't find " + RUNTIME + " bean");
            }
            if (this.threadingMBeanName == null) {
                throw new SystemMonitoringDoesNotExistException("Can't find " + THREADING + " bean");
            }
            if (this.memoryMBeanName == null) {
                throw new SystemMonitoringDoesNotExistException("Can't find " + MEMORY + " bean");
            }
            if (this.classLoadingMBeanName == null) {
                throw new SystemMonitoringDoesNotExistException("Can't find " + CLASS_LOADING
                        + " bean");
            }

        } catch (MalformedObjectNameException e) {
            throw new SystemMonitoringServiceErrorException(e);
        } catch (IOException e) {
            throw new SystemMonitoringServiceErrorException(e);
        }
    }


    /**
     * Get heap memory usage
     * 
     * @throws SystemMonitoringServiceErrorException
     */
    public CompositeDataSupport getHeapMemoryUsage() throws SystemMonitoringServiceErrorException {

        Object result;
        try {
            result = this.getAttribute(this.memoryMBeanName, HEAP_MEMORY_USAGE);

            if (result == null) {
                throw new PerformActionErrorException("Unexpected result : null");
            } else if (!(result instanceof CompositeDataSupport)) {
                throw new PerformActionErrorException("Unexpected result type");
            } else {
                return ((CompositeDataSupport) result);
            }
        } catch (AttributeErrorException e) {
            throw new SystemMonitoringServiceErrorException(e);
        } catch (final PerformActionErrorException e) {
            throw new SystemMonitoringServiceErrorException(e);
        }

    }

    /**
     * Get heap memory usage
     * 
     * @throws SystemMonitoringServiceErrorException
     */
    public int getPendingObjects() throws SystemMonitoringServiceErrorException {

        Object result;
        try {
            result = this.getAttribute(this.memoryMBeanName, OBJECT_PENDING_FINALIZATION_COUNT);

            if (result == null) {
                throw new PerformActionErrorException("Unexpected result : null");
            } else if (!(result instanceof Integer)) {
                throw new PerformActionErrorException("Unexpected result type");
            } else {
                return ((Integer) result).intValue();
            }
        } catch (AttributeErrorException e) {
            throw new SystemMonitoringServiceErrorException(e);
        } catch (final PerformActionErrorException e) {
            throw new SystemMonitoringServiceErrorException(e);
        }

    }

    /**
     * Get Peak thread count
     * 
     * @throws SystemMonitoringServiceErrorException
     */
    public int getPeakThreadCount() throws SystemMonitoringServiceErrorException {

        Object result;
        try {
            result = this.getAttribute(this.threadingMBeanName, PEAK_THREAD_COUNT);

            if (result == null) {
                throw new PerformActionErrorException("Unexpected result : null");
            } else if (!(result instanceof Integer)) {
                throw new PerformActionErrorException("Unexpected result type");
            } else {
                return ((Integer) result).intValue();
            }
        } catch (AttributeErrorException e) {
            throw new SystemMonitoringServiceErrorException(e);
        } catch (final PerformActionErrorException e) {
            throw new SystemMonitoringServiceErrorException(e);
        }

    }

    /**
     * Get Daemon thread count
     * 
     * @throws SystemMonitoringServiceErrorException
     */
    public int getDaemonThreadCount() throws SystemMonitoringServiceErrorException {

        Object result;
        try {
            result = this.getAttribute(this.threadingMBeanName, DAEMON_THREAD_COUNT);

            if (result == null) {
                throw new PerformActionErrorException("Unexpected result : null");
            } else if (!(result instanceof Integer)) {
                throw new PerformActionErrorException("Unexpected result type");
            } else {
                return ((Integer) result).intValue();
            }
        } catch (AttributeErrorException e) {
            throw new SystemMonitoringServiceErrorException(e);
        } catch (final PerformActionErrorException e) {
            throw new SystemMonitoringServiceErrorException(e);
        }
    }

    /**
     * Get thread count
     * 
     * @throws SystemMonitoringServiceErrorException
     */
    public int getThreadCount() throws SystemMonitoringServiceErrorException {

        Object result;
        try {
            result = this.getAttribute(this.threadingMBeanName, THREAD_COUNT);

            if (result == null) {
                throw new PerformActionErrorException("Unexpected result : null");
            } else if (!(result instanceof Integer)) {
                throw new PerformActionErrorException("Unexpected result type");
            } else {
                return ((Integer) result).intValue();
            }
        } catch (AttributeErrorException e) {
            throw new SystemMonitoringServiceErrorException(e);
        } catch (PerformActionErrorException e) {
            throw new SystemMonitoringServiceErrorException(e);
        }
    }

    /**
     * Get the total started thread count
     * 
     * @throws SystemMonitoringServiceErrorException
     */
    public long getTotalStartedThreadCount() throws SystemMonitoringServiceErrorException {

        Object result;
        try {
            result = this.getAttribute(this.threadingMBeanName, TOTAL_STARTED_THREAD_COUNT);

            if (result == null) {
                throw new PerformActionErrorException("Unexpected result : null");
            } else if (!(result instanceof Long)) {
                throw new PerformActionErrorException("Unexpected result type");
            } else {
                return ((Long) result).longValue();
            }
        } catch (AttributeErrorException e) {
            throw new SystemMonitoringServiceErrorException(e);
        } catch (final PerformActionErrorException e) {
            throw new SystemMonitoringServiceErrorException(e);
        }
    }

    /**
     * Get the available processors
     * 
     * @throws SystemMonitoringServiceErrorException
     */
    public int getAvailableProcessors() throws SystemMonitoringServiceErrorException {

        Object result;
        try {
            result = this.getAttribute(this.operatingSystemMBeanName, AVAILABLE_PROCESSORS);

            if (result == null) {
                throw new PerformActionErrorException("Unexpected result : null");
            } else if (!(result instanceof Integer)) {
                throw new PerformActionErrorException("Unexpected result type");
            } else {
                return ((Integer) result).intValue();
            }
        } catch (AttributeErrorException e) {
            throw new SystemMonitoringServiceErrorException(e);
        } catch (final PerformActionErrorException e) {
            throw new SystemMonitoringServiceErrorException(e);
        }
    }

    /**
     * Get the up time
     * 
     * @throws SystemMonitoringServiceErrorException
     */
    public long getUptime() throws SystemMonitoringServiceErrorException {

        Object result;
        try {
            result = this.getAttribute(this.runtimeMBeanName, UP_TIME);

            if (result == null) {
                throw new PerformActionErrorException("Unexpected result : null");
            } else if (!(result instanceof Long)) {
                throw new PerformActionErrorException("Unexpected result type");
            } else {
                return ((Long) result).longValue();
            }
        } catch (AttributeErrorException e) {
            throw new SystemMonitoringServiceErrorException(e);
        } catch (final PerformActionErrorException e) {
            throw new SystemMonitoringServiceErrorException(e);
        }
    }

    /**
     * Get the loaded class count
     * 
     * @throws SystemMonitoringServiceErrorException
     */
    public int getLoadedClassCount() throws SystemMonitoringServiceErrorException {

        Object result;
        try {
            result = this.getAttribute(this.classLoadingMBeanName, LOADED_CLASS_COUNT);

            if (result == null) {
                throw new PerformActionErrorException("Unexpected result : null");
            } else if (!(result instanceof Integer)) {
                throw new PerformActionErrorException("Unexpected result type");
            } else {
                return ((Integer) result).intValue();
            }
        } catch (AttributeErrorException e) {
            throw new SystemMonitoringServiceErrorException(e);
        } catch (final PerformActionErrorException e) {
            throw new SystemMonitoringServiceErrorException(e);
        }
    }

    /**
     * Get the total loaded class count
     * 
     * @throws SystemMonitoringServiceErrorException
     */
    public long getTotalLoadedClassCount() throws SystemMonitoringServiceErrorException {

        Object result;
        try {
            result = this.getAttribute(this.classLoadingMBeanName, TOTAL_LOADED_CLASS_COUNT);

            if (result == null) {
                throw new PerformActionErrorException("Unexpected result : null");
            } else if (!(result instanceof Long)) {
                throw new PerformActionErrorException("Unexpected result type");
            } else {
                return ((Long) result).longValue();
            }
        }catch (AttributeErrorException e) {
            throw new SystemMonitoringServiceErrorException(e);
        } catch (final PerformActionErrorException e) {
            throw new SystemMonitoringServiceErrorException(e);
        }
    }

    /**
     * Get the unloaded class count
     * 
     * @throws SystemMonitoringServiceErrorException
     */
    public long getUnloadedClassCount() throws SystemMonitoringServiceErrorException {

        Object result;
        try {
            result = this.getAttribute(this.classLoadingMBeanName, UNLOADED_CLASS_COUNT);

            if (result == null) {
                throw new PerformActionErrorException("Unexpected result : null");
            } else if (!(result instanceof Long)) {
                throw new PerformActionErrorException("Unexpected result type");
            } else {
                return ((Long) result).longValue();
            }
        }catch (AttributeErrorException e) {
            throw new SystemMonitoringServiceErrorException(e);
        } catch (final PerformActionErrorException e) {
            throw new SystemMonitoringServiceErrorException(e);
        }
    }

    /**
     * Get the process cpu time
     * 
     * @throws SystemMonitoringServiceErrorException
     */
    public long getProcessCpuTime() throws SystemMonitoringServiceErrorException {

        Object result;
        try {
            result = this.getAttribute(this.operatingSystemMBeanName, PROCESS_CPU_TIME);

            if (result == null) {
                throw new PerformActionErrorException("Unexpected result : null");
            } else if (!(result instanceof Long)) {
                throw new PerformActionErrorException("Unexpected result type");
            } else {
                return ((Long) result).longValue();
            }
        } catch (AttributeErrorException e) {
            throw new SystemMonitoringServiceErrorException(e);
        } catch (final PerformActionErrorException e) {
            throw new SystemMonitoringServiceErrorException(e);
        }
    }

    /**
     * Get the total physical mem size
     * 
     * @throws SystemMonitoringServiceErrorException
     */
    public long getTotalPhysicalMemorySize() throws SystemMonitoringServiceErrorException {

        Object result;
        try {
            result = this.getAttribute(this.operatingSystemMBeanName, TOTAL_PHYSICAL_MEMORY_SIZE);

            if (result == null) {
                throw new PerformActionErrorException("Unexpected result : null");
            } else if (!(result instanceof Long)) {
                throw new PerformActionErrorException("Unexpected result type");
            } else {
                return ((Long) result).longValue();
            }
        } catch (AttributeErrorException e) {
            throw new SystemMonitoringServiceErrorException(e);
        } catch (final PerformActionErrorException e) {
            throw new SystemMonitoringServiceErrorException(e);
        }
    }

    /**
     * Get the free physical memory size
     * 
     * @throws SystemMonitoringServiceErrorException
     */
    public long getFreePhysicalMemorySize() throws SystemMonitoringServiceErrorException {

        Object result;
        try {
            result = this.getAttribute(this.operatingSystemMBeanName, FREE_PHYSICAL_MEMORY_SIZE);

            if (result == null) {
                throw new PerformActionErrorException("Unexpected result : null");
            } else if (!(result instanceof Long)) {
                throw new PerformActionErrorException("Unexpected result type");
            } else {
                return ((Long) result).longValue();
            }
        } catch (AttributeErrorException e) {
            throw new SystemMonitoringServiceErrorException(e);
        } catch (final PerformActionErrorException e) {
            throw new SystemMonitoringServiceErrorException(e);
        }
    }

    /**
     * Get the committed physical memory size
     * 
     * @throws SystemMonitoringServiceErrorException
     */
    public long getCommittedVirtualMemorySize() throws SystemMonitoringServiceErrorException {

        Object result;
        try {
            result = this.getAttribute(this.operatingSystemMBeanName, COMMITTED_VIRTUAL_MEMORY_SIZE);

            if (result == null) {
                throw new PerformActionErrorException("Unexpected result : null");
            } else if (!(result instanceof Long)) {
                throw new PerformActionErrorException("Unexpected result type");
            } else {
                return ((Long) result).longValue();
            }
        } catch (AttributeErrorException e) {
            throw new SystemMonitoringServiceErrorException(e);
        } catch (final PerformActionErrorException e) {
            throw new SystemMonitoringServiceErrorException(e);
        }
    }

}
