/**
 * Copyright (c) 2013-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl;

import java.io.IOException;
import java.util.Hashtable;
import java.util.Map;
import java.util.Set;

import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.clientserverapi.topology.RemoteContainer;
import org.ow2.petals.clientserverapi.topology.exception.AttachContainerException;
import org.ow2.petals.clientserverapi.topology.exception.DetachContainerException;
import org.ow2.petals.clientserverapi.topology.exception.DomainIsNotDynamicException;
import org.ow2.petals.clientserverapi.topology.exception.InconsistencyException;
import org.ow2.petals.clientserverapi.topology.exception.NoDomainConfigurationProvidedException;
import org.ow2.petals.clientserverapi.topology.exception.NoRemoteContainerProvidedException;
import org.ow2.petals.jmx.api.api.InstallationServiceClient;
import org.ow2.petals.jmx.api.api.TopologyServiceClient;
import org.ow2.petals.jmx.api.api.exception.InstallationServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.InstallationServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.TopologyServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.TopologyServiceErrorException;
import org.ow2.petals.jmx.api.impl.exception.PerformActionDoesNotExistException;
import org.ow2.petals.jmx.api.impl.exception.PerformActionErrorException;
import org.ow2.petals.jmx.api.impl.exception.PerformActionRemoteErrorException;

/**
 * Client API of the JMX topology service.
 * 
 * @author Christophe Deneux - Linagora
 */
public final class TopologyServiceClientImpl extends AbstractServiceClient implements
        TopologyServiceClient {

    /**
     * Action : attachContainerTo
     */
    private static final String ATTACH_CONTAINER = "attachContainerTo";

    /**
     * Action : detachContainer
     */
    private static final String DETACH_CONTAINER = "detachContainer";

    /**
     * Action : retrieve topology
     */
    public static final String RETRIEVE_TOPOLOGY = "retrieveTopology";

    /**
     * Action : retrieve topology object
     */
    public static final String RETRIEVE_TOPOLOGY_OBJECT = "retrieveTopologyObject";

    /**
     * Action : get shared area client implementation
     */
    public static final String RETRIEVE_SHARED_AREA_IMPL = "retrieveSharedAreaCltImplementation";

    /**
     * Creates a new instance of {@link InstallationServiceClient}
     * 
     * @param petalsDomain
     * @param mBeanServerConnection
     * @throws InstallationServiceDoesNotExistException
     * @throws InstallationServiceErrorException
     */
    public TopologyServiceClientImpl(String petalsDomain,
            MBeanServerConnection mBeanServerConnection)
            throws TopologyServiceDoesNotExistException, TopologyServiceErrorException {

        super(petalsDomain, mBeanServerConnection);

        try {
            final Hashtable<String, String> attributes = new Hashtable<>();
            attributes.put(JMX_FIELD_NAME, TopologyServiceClient.TOPOLOGY_SERVICE_MBEAN_NAME);
            attributes.put(JMX_FIELD_TYPE, JMX_SERVICE_TYPE);
            final ObjectName objName = new ObjectName(petalsDomain, attributes);
            final Set<ObjectName> objNames = this.mBeanServerConnection.queryNames(objName, null);
            if (objNames != null && objNames.size() == 1) {
                this.mbeanName = objNames.iterator().next();
            } else {
                throw new TopologyServiceDoesNotExistException(objName.toString());
            }
        } catch (final MalformedObjectNameException e) {
            throw new TopologyServiceErrorException(e);
        } catch (final IOException e) {
            throw new TopologyServiceErrorException(e);
        }
    }

    @Override
    public Set<Map<String, String>> retrieveTopology(final String securityPassPhrase,
            final boolean isSensibleInformationExpected) throws PetalsException {

        final Object[] objects = new Object[] { securityPassPhrase, isSensibleInformationExpected };
        final String[] strings = new String[] { String.class.getName(), boolean.class.getName() };

        try {
            return (Set<Map<String, String>>) this.performAction(RETRIEVE_TOPOLOGY, objects, strings);
        } catch (PerformActionErrorException e) {
            throw new TopologyServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new TopologyServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public Object retrieveTopologyObject(final String securityPassPhrase, final boolean isSensibleInformationExpected)
            throws PetalsException {

        final Object[] objects = new Object[] { securityPassPhrase, isSensibleInformationExpected };
        final String[] strings = new String[] { String.class.getName(), boolean.class.getName() };

        try {
            return this.performAction(RETRIEVE_TOPOLOGY_OBJECT, objects, strings);
        } catch (PerformActionErrorException e) {
            throw new TopologyServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new TopologyServiceErrorException("Unexpected result type", e);
        }
    }

    /**
     * <p>
     * JMX invocation errors are reported as {@link AttachContainerException}.
     * </p>
     */
    @Override
    public void attachContainerTo(final RemoteContainer container, final String securityPassPhrase)
            throws NoRemoteContainerProvidedException, NoDomainConfigurationProvidedException,
            DomainIsNotDynamicException, InconsistencyException, AttachContainerException {

        final Object[] objects = new Object[] { container, securityPassPhrase };
        final String[] types = new String[] { RemoteContainer.class.getName(), String.class.getName() };

        try {
            this.performAction(ATTACH_CONTAINER, objects, types);
        } catch (final PerformActionDoesNotExistException e) {
            throw new AttachContainerException(container, e);
        } catch (final PerformActionRemoteErrorException e) {
            throw new AttachContainerException(container, e);
        } catch (final PerformActionErrorException e) {
            throw new AttachContainerException(container, e);
        }

    }

    /**
     * <p>
     * JMX invocation errors are reported as {@link DetachContainerException}.
     * </p>
     */
    @Override
    public void detachContainer() throws DetachContainerException {

        final Object[] objects = new Object[] {};
        final String[] types = new String[] {};

        try {
            this.performAction(DETACH_CONTAINER, objects, types);
        } catch (final PerformActionDoesNotExistException e) {
            throw new DetachContainerException(e);
        } catch (final PerformActionRemoteErrorException e) {
            throw new DetachContainerException(e);
        } catch (final PerformActionErrorException e) {
            throw new DetachContainerException(e);
        }
    }
}
