/**
 * Copyright (c) 2013-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl.monitoring.component.framework;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.ow2.petals.jmx.api.api.exception.AttributeErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.ComponentMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.ServiceProviderInvocationKey;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.ServiceProviderInvocationStatus;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.api.monitoring.object.ThreadPoolMetrics;
import org.ow2.petals.jmx.api.impl.AbstractServiceClient;
import org.ow2.petals.jmx.api.impl.monitoring.object.ThreadPoolMetricsImpl;

/**
 * Abstract client API of the JMX service providing CDK monitoring metrics .
 * 
 * @author Christophe Deneux - Linagora
 */
public class ComponentMonitoringServiceClientImpl extends AbstractServiceClient
        implements ComponentMonitoringServiceClient {

    /**
     * Attribute : ServiceProviderInvocations
     */
    private static final String SERVICE_PROVIDER_INVOCATIONS_COUNT = "ServiceProviderInvocations";

    /**
     * Attribute : ServiceProviderInvocationsResponseTimeRel
     */
    private static final String SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIME_REL = "ServiceProviderInvocationsResponseTimeRel";

    /**
     * Attribute : ServiceProviderInvocationsResponseTimeRelGroupedByItfSvcOpAndExecStatus
     */
    private static final String SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIME_REL_BY_ITF_SVC_OP_AND_EXEC_STATUS = "ServiceProviderInvocationsResponseTimeRelGroupedByItfSvcOpAndExecStatus";

    /**
     * Attribute : MessageExchangeAcceptorCurrentWorking
     */
    private static final String MEX_ACCEPTOR_WORKING_CURRENT = "MessageExchangeAcceptorCurrentWorking";

    /**
     * Attribute : MessageExchangeAcceptorCurrentWorking
     */
    private static final String MEX_ACCEPTOR_WORKING_MAX = "MessageExchangeAcceptorMaxWorking";

    /**
     * Attribute : MessageExchangeAcceptorRelativeDurations
     */
    private static final String MEX_ACCEPTOR_WORKING_RELATIVE_DURATIONS = "MessageExchangeAcceptorRelativeDurations";

    /**
     * Attribute : MessageExchangeAcceptorThreadPoolCurrentSize
     */
    private static final String MEX_ACCEPTOR_THREAD_POOL_ACTIVE_THREADS_CURRENT = "MessageExchangeAcceptorThreadPoolCurrentSize";

    /**
     * Attribute : MessageExchangeAcceptorThreadPoolMaxSize
     */
    private static final String MEX_ACCEPTOR_THREAD_POOL_ACTIVE_THREADS_MAX = "MessageExchangeAcceptorThreadPoolMaxSize";

    /**
     * Attribute : MessageExchangeProcessorThreadPoolMinSize
     */
    private static final String MEX_PROCESSOR_THREAD_POOL_MIN_SIZE = "MessageExchangeProcessorThreadPoolMinSize";

    /**
     * Attribute : MessageExchangeProcessorThreadPoolMaxSize
     */
    private static final String MEX_PROCESSOR_THREAD_POOL_MAX_SIZE = "MessageExchangeProcessorThreadPoolMaxSize";

    /**
     * Attribute : MessageExchangeProcessorThreadPoolActiveThreadsCurrent
     */
    private static final String MEX_PROCESSOR_THREAD_POOL_ACTIVE_THREADS_CURRENT = "MessageExchangeProcessorThreadPoolActiveThreadsCurrent";

    /**
     * Attribute : MessageExchangeProcessorThreadPoolActiveThreadsMax
     */
    private static final String MEX_PROCESSOR_THREAD_POOL_ACTIVE_THREADS_MAX = "MessageExchangeProcessorThreadPoolActiveThreadsMax";

    /**
     * Attribute : MessageExchangeProcessorThreadPoolIdleThreadsCurrent
     */
    private static final String MEX_PROCESSOR_THREAD_POOL_IDLE_THREADS_CURRENT = "MessageExchangeProcessorThreadPoolIdleThreadsCurrent";

    /**
     * Attribute : MessageExchangeProcessorThreadPoolIdleThreadsMax
     */
    private static final String MEX_PROCESSOR_THREAD_POOL_IDLE_THREADS_MAX = "MessageExchangeProcessorThreadPoolIdleThreadsMax";

    /**
     * Attribute : MessageExchangeProcessorThreadPoolQueuedRequestsCurrent
     */
    private static final String MEX_PROCESSOR_THREAD_POOL_QUEUED_REQUESTS_CURRENT = "MessageExchangeProcessorThreadPoolQueuedRequestsCurrent";

    /**
     * Attribute : MessageExchangeProcessorThreadPoolQueuedRequestsMax
     */
    private static final String MEX_PROCESSOR_THREAD_POOL_QUEUED_REQUESTS_MAX = "MessageExchangeProcessorThreadPoolQueuedRequestsMax";

    /**
     * JMX base name
     */
    protected static final String SERVICE_JMX_BASENAME = "monitoring_";

    /**
     * Creates a new instance of {@link ComponentMonitoringServiceClient}
     * 
     * @param petalsDomain
     * @param mBeanServerConnection
     * @throws ComponentMonitoringServiceDoesNotExistException
     * @throws ComponentMonitoringServiceErrorException
     */
    public ComponentMonitoringServiceClientImpl(final String petalsDomain, final String componentIdentifier,
            final MBeanServerConnection mBeanServerConnection)
                    throws ComponentMonitoringServiceDoesNotExistException, ComponentMonitoringServiceErrorException {

        super(petalsDomain, mBeanServerConnection);

        try {
            final String pattern = "*:type=custom,name=" + SERVICE_JMX_BASENAME + componentIdentifier;
            final ObjectName objName = new ObjectName(pattern);
            final Set<ObjectName> objNames = this.mBeanServerConnection.queryNames(objName, null);
            if (objNames != null && objNames.size() == 1) {
                this.mbeanName = objNames.iterator().next();
            } else {
                throw new ComponentMonitoringServiceDoesNotExistException();
            }
        } catch (final MalformedObjectNameException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final IOException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        }
    }

    @Override
    public Map<ServiceProviderInvocationKey, Long> getServiceProviderInvocationsCount()
            throws ComponentMonitoringServiceErrorException {

        try {
            final Map<String[], Long> values = (Map<String[], Long>) this
                    .getAttribute(SERVICE_PROVIDER_INVOCATIONS_COUNT);

            final Map<ServiceProviderInvocationKey, Long> results = new HashMap<ServiceProviderInvocationKey, Long>(
                    values.size());
            for (final Map.Entry<String[], Long> entry : values.entrySet()) {
                final String[] key = entry.getKey();
                results.put(
                        new ServiceProviderInvocationKey(key[ServiceProviderKeyIndexes.INTERFACE_IDX],
                                key[ServiceProviderKeyIndexes.SERVICE_IDX],
                                key[ServiceProviderKeyIndexes.OPERATION_IDX], key[ServiceProviderKeyIndexes.MEP_IDX],
                                ServiceProviderInvocationStatus.parse(key[ServiceProviderKeyIndexes.EXEC_STATUS_IDX])),
                        entry.getValue());
            }
            return results;

        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public Map<ServiceProviderInvocationKey, Long[]> getServiceProviderInvocationsResponseTimes()
            throws ComponentMonitoringServiceErrorException {

        try {
            final Map<String[], Long[]> values = (Map<String[], Long[]>) this
                    .getAttribute(SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIME_REL);

            final Map<ServiceProviderInvocationKey, Long[]> results = new HashMap<ServiceProviderInvocationKey, Long[]>(
                    values.size());
            for (final Map.Entry<String[], Long[]> entry : values.entrySet()) {
                final String[] key = entry.getKey();
                results.put(
                        new ServiceProviderInvocationKey(key[ServiceProviderKeyIndexes.INTERFACE_IDX],
                                key[ServiceProviderKeyIndexes.SERVICE_IDX],
                                key[ServiceProviderKeyIndexes.OPERATION_IDX], key[ServiceProviderKeyIndexes.MEP_IDX],
                                ServiceProviderInvocationStatus.parse(key[ServiceProviderKeyIndexes.EXEC_STATUS_IDX])),
                        entry.getValue());
            }
            return results;
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public Map<ServiceProviderInvocationKey, Long[]> getServiceProviderInvocationsResponseTimesGroupedByItfSvcOpAndExecStatus()
            throws ComponentMonitoringServiceErrorException {

        try {
            final Map<String[], Long[]> values = (Map<String[], Long[]>) this
                    .getAttribute(SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIME_REL_BY_ITF_SVC_OP_AND_EXEC_STATUS);

            final Map<ServiceProviderInvocationKey, Long[]> results = new HashMap<ServiceProviderInvocationKey, Long[]>(
                    values.size());
            for (final Map.Entry<String[], Long[]> entry : values.entrySet()) {
                final String[] key = entry.getKey();
                results.put(new ServiceProviderInvocationKey(key[0], key[1], key[2], null,
                        ServiceProviderInvocationStatus.parse(key[3])), entry.getValue());
            }
            return results;
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public Integer getMexAcceptorThreadPoolActiveThreadsMaxSize() throws ComponentMonitoringServiceErrorException {

        try {
            return (Integer) this.getAttribute(MEX_ACCEPTOR_THREAD_POOL_ACTIVE_THREADS_MAX);
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public Integer getMexAcceptorThreadPoolActiveThreadsCurrentSize() throws ComponentMonitoringServiceErrorException {

        try {
            return (Integer) this.getAttribute(MEX_ACCEPTOR_THREAD_POOL_ACTIVE_THREADS_CURRENT);
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public ThreadPoolMetrics getMexProcessorThreadPoolMetrics() throws ComponentMonitoringServiceErrorException {

        return this.getThreadPoolMetrics(MEX_PROCESSOR_THREAD_POOL_MIN_SIZE, MEX_PROCESSOR_THREAD_POOL_MAX_SIZE,
                MEX_PROCESSOR_THREAD_POOL_ACTIVE_THREADS_CURRENT, MEX_PROCESSOR_THREAD_POOL_ACTIVE_THREADS_MAX,
                MEX_PROCESSOR_THREAD_POOL_IDLE_THREADS_CURRENT, MEX_PROCESSOR_THREAD_POOL_IDLE_THREADS_MAX,
                MEX_PROCESSOR_THREAD_POOL_QUEUED_REQUESTS_CURRENT, MEX_PROCESSOR_THREAD_POOL_QUEUED_REQUESTS_MAX);
    }

    @Override
    public Long getMexAcceptorWorkingCurrent() throws ComponentMonitoringServiceErrorException {
        try {
            return (Long) this.getAttribute(MEX_ACCEPTOR_WORKING_CURRENT);
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public Long getMexAcceptorWorkingMax() throws ComponentMonitoringServiceErrorException {
        try {
            return (Long) this.getAttribute(MEX_ACCEPTOR_WORKING_MAX);
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public Long[] getMexAcceptorRelativeDurations() throws ComponentMonitoringServiceErrorException {
        try {
            return (Long[]) this.getAttribute(MEX_ACCEPTOR_WORKING_RELATIVE_DURATIONS);
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type", e);
        }
    }

    /**
     * Build a {@link ThreadPoolMetrics} from given JMX attribute names
     * 
     * @param attrMinSize
     *            Name of the JMX attribute associated to the thread pool min size
     * @param attrMaxSize
     *            Name of the JMX attribute associated to the thread pool max size
     * @param attrActiveCurrent
     *            Name of the JMX attribute associated to the current number of active threads of the thread pool
     * @param attrActiveMax
     *            Name of the JMX attribute associated to the max number of active threads of the thread pool
     * @param attrIdleCurrent
     *            Name of the JMX attribute associated to the current number of idle threads of the thread pool
     * @param attrIdleMax
     *            Name of the JMX attribute associated to the max number of idle threads of the thread pool
     * @param attrQueuedRequestsCurrent
     *            Name of the JMX attribute associated to the current number of enqueued request of the thread pool
     * @param attrQueuedRequestsMax
     *            Name of the JMX attribute associated to the max number of enqueued request of the thread pool
     * @return The {@link ThreadPoolMetrics}
     * @throws ComponentMonitoringServiceErrorException
     */
    protected ThreadPoolMetrics getThreadPoolMetrics(final String attrMinSize, final String attrMaxSize,
            final String attrActiveCurrent, final String attrActiveMax, final String attrIdleCurrent,
            final String attrIdleMax, final String attrQueuedRequestsCurrent, final String attrQueuedRequestsMax)
            throws ComponentMonitoringServiceErrorException {

        try {
            final ThreadPoolMetricsImpl asyncExecutorThreadPoolMetrics = new ThreadPoolMetricsImpl();

            // Min size
            asyncExecutorThreadPoolMetrics.setMinSize((Long) this.getAttribute(attrMinSize));
            // Max size
            asyncExecutorThreadPoolMetrics.setMaxSize((Long) this.getAttribute(attrMaxSize));
            // Active thread current size
            asyncExecutorThreadPoolMetrics
                    .setActiveThreadsSize((Long) this.getAttribute(attrActiveCurrent));
            // Active thread max size
            asyncExecutorThreadPoolMetrics
                    .setActiveThreadsMaxSize((Long) this.getAttribute(attrActiveMax));
            // Idle thread size
            asyncExecutorThreadPoolMetrics
                    .setIdleThreadsSize((Long) this.getAttribute(attrIdleCurrent));
            // Idle thread max size
            asyncExecutorThreadPoolMetrics
                    .setIdleThreadsMaxSize((Long) this.getAttribute(attrIdleMax));
            // Enqueued Size
            asyncExecutorThreadPoolMetrics.setEnqueuedRequestsSize(
                    (Long) this.getAttribute(attrQueuedRequestsCurrent));
            // Enqueued Max Size
            asyncExecutorThreadPoolMetrics.setEnqueuedRequestsMaxSize(
                    (Long) this.getAttribute(attrQueuedRequestsMax));

            return asyncExecutorThreadPoolMetrics;
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type", e);
        }
    }

}
