/**
 * Copyright (c) 2016-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl.monitoring.component.se.flowable;

import java.util.Map;

import javax.management.MBeanServerConnection;

import org.ow2.petals.jmx.api.api.exception.AttributeErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.se.flowable.DatabaseConnectionPoolMetrics;
import org.ow2.petals.jmx.api.api.monitoring.component.se.flowable.SeFlowableMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.monitoring.object.ThreadPoolMetrics;
import org.ow2.petals.jmx.api.impl.monitoring.component.framework.ComponentMonitoringServiceClientImpl;

/**
 * Client API of the JMX service providing Petals SE Flowable monitoring metrics.
 * 
 * @author Christophe Deneux - Linagora
 */
public final class SeFlowableMonitoringServiceClientImpl extends ComponentMonitoringServiceClientImpl
        implements SeFlowableMonitoringServiceClient {

    /**
     * Attribute : ProcessDefinitions
     */
    private static final String PROCESS_DEFINITIONS = "ProcessDefinitions";

    /**
     * Attribute : AsyncExecutorThreadPoolMinSize
     */
    private static final String ASYNC_EXECUTOR_THREAD_POOL_MIN_SIZE = "AsyncExecutorThreadPoolMinSize";

    /**
     * Attribute : AsyncExecutorThreadPoolMaxSize
     */
    private static final String ASYNC_EXECUTOR_THREAD_POOL_MAX_SIZE = "AsyncExecutorThreadPoolMaxSize";

    /**
     * Attribute : AsyncExecutorThreadPoolActiveThreadsCurrent
     */
    private static final String ASYNC_EXECUTOR_THREAD_POOL_ACTIVE_THREADS_CURRENT = "AsyncExecutorThreadPoolActiveThreadsCurrent";

    /**
     * Attribute : AsyncExecutorThreadPoolActiveThreadsMax
     */
    private static final String ASYNC_EXECUTOR_THREAD_POOL_ACTIVE_THREADS_MAX = "AsyncExecutorThreadPoolActiveThreadsMax";

    /**
     * Attribute : AsyncExecutorThreadPoolIdleThreadsCurrent
     */
    private static final String ASYNC_EXECUTOR_THREAD_POOL_IDLE_THREADS_CURRENT = "AsyncExecutorThreadPoolIdleThreadsCurrent";

    /**
     * Attribute : AsyncExecutorThreadPoolIdleThreadsMax
     */
    private static final String ASYNC_EXECUTOR_THREAD_POOL_IDLE_THREADS_MAX = "AsyncExecutorThreadPoolIdleThreadsMax";

    /**
     * Attribute : AsyncExecutorThreadPoolQueuedRequestsCurrent
     */
    private static final String ASYNC_EXECUTOR_THREAD_POOL_QUEUED_REQUESTS_CURRENT = "AsyncExecutorThreadPoolQueuedRequestsCurrent";

    /**
     * Attribute : AsyncExecutorThreadPoolQueuedRequestsMax
     */
    private static final String ASYNC_EXECUTOR_THREAD_POOL_QUEUED_REQUESTS_MAX = "AsyncExecutorThreadPoolQueuedRequestsMax";

    /**
     * Attribute : DatabaseConnectionPoolMaxActiveSize
     */
    private static final String DATABASE_CONNECTION_POOL_MAX_ACTIVE_SIZE = "DatabaseConnectionPoolMaxActiveSize";

    /**
     * Attribute : DatabaseConnectionPoolMaxIdleSize
     */
    private static final String DATABASE_CONNECTION_POOL_MAX_IDLE_SIZE = "DatabaseConnectionPoolMaxIdleSize";

    /**
     * Attribute : DatabaseConnectionPoolActiveConnectionsCurrent
     */
    private static final String DATABASE_CONNECTION_POOL_ACTIVE_CONNECTIONS_CURRENT = "DatabaseConnectionPoolActiveConnectionsCurrent";

    /**
     * Attribute : DatabaseConnectionPoolActiveConnectionsMax
     */
    private static final String DATABASE_CONNECTION_POOL_ACTIVE_CONNECTIONS_MAX = "DatabaseConnectionPoolActiveConnectionsMax";

    /**
     * Attribute : DatabaseConnectionPoolIdleConnectionsCurrent
     */
    private static final String DATABASE_CONNECTION_POOL_IDLE_CONNECTIONS_CURRENT = "DatabaseConnectionPoolIdleConnectionsCurrent";

    /**
     * Attribute : DatabaseConnectionPoolIdleConnectionsMax
     */
    private static final String DATABASE_CONNECTION_POOL_IDLE_CONNECTIONS_MAX = "DatabaseConnectionPoolIdleConnectionsMax";

    /**
     * Creates a new instance of {@link SeFlowableMonitoringServiceClient}
     * 
     * @param petalsDomain
     * @param componentIdentifier
     *            Component unique identifier
     * @param mBeanServerConnection
     * @throws ComponentMonitoringServiceDoesNotExistException
     * @throws ComponentMonitoringServiceErrorException
     */
    public SeFlowableMonitoringServiceClientImpl(final String petalsDomain, final String componentIdentifier,
            final MBeanServerConnection mBeanServerConnection)
            throws ComponentMonitoringServiceDoesNotExistException, ComponentMonitoringServiceErrorException {

        super(petalsDomain, componentIdentifier, mBeanServerConnection);
    }

    @Override
    public ThreadPoolMetrics getAsyncExecutorThreadPoolMetrics() throws ComponentMonitoringServiceErrorException {

        return this.getThreadPoolMetrics(ASYNC_EXECUTOR_THREAD_POOL_MIN_SIZE, ASYNC_EXECUTOR_THREAD_POOL_MAX_SIZE,
                ASYNC_EXECUTOR_THREAD_POOL_ACTIVE_THREADS_CURRENT, ASYNC_EXECUTOR_THREAD_POOL_ACTIVE_THREADS_MAX,
                ASYNC_EXECUTOR_THREAD_POOL_IDLE_THREADS_CURRENT, ASYNC_EXECUTOR_THREAD_POOL_IDLE_THREADS_MAX,
                ASYNC_EXECUTOR_THREAD_POOL_QUEUED_REQUESTS_CURRENT, ASYNC_EXECUTOR_THREAD_POOL_QUEUED_REQUESTS_MAX);
    }

    @Override
    public DatabaseConnectionPoolMetrics getDatabaseConnectionPoolMetrics()
            throws ComponentMonitoringServiceErrorException {
        try {
            final DatabaseConnectionPoolMetricsImpl databaseConnectionPoolMetrics = new DatabaseConnectionPoolMetricsImpl();

            // Max active size
            databaseConnectionPoolMetrics
                    .setMaxActiveSize((Long) this.getAttribute(DATABASE_CONNECTION_POOL_MAX_ACTIVE_SIZE));
            // Max idle size
            databaseConnectionPoolMetrics
                    .setMaxIdleSize((Long) this.getAttribute(DATABASE_CONNECTION_POOL_MAX_IDLE_SIZE));
            // Active connections current size
            databaseConnectionPoolMetrics.setActiveConnectionsSize(
                    (Long) this.getAttribute(DATABASE_CONNECTION_POOL_ACTIVE_CONNECTIONS_CURRENT));
            // Active connection max size
            databaseConnectionPoolMetrics.setActiveConnectionsMaxSize(
                    (Long) this.getAttribute(DATABASE_CONNECTION_POOL_ACTIVE_CONNECTIONS_MAX));
            // Idle connection size
            databaseConnectionPoolMetrics.setIdleConnectionsSize(
                    (Long) this.getAttribute(DATABASE_CONNECTION_POOL_IDLE_CONNECTIONS_CURRENT));
            // Idle connection max size
            databaseConnectionPoolMetrics
                    .setIdleConnectionsMaxSize((Long) this.getAttribute(DATABASE_CONNECTION_POOL_IDLE_CONNECTIONS_MAX));

            return databaseConnectionPoolMetrics;
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public Map<String, Long[]> getProcessDefinitions() throws ComponentMonitoringServiceErrorException {

        try {
            return (Map<String, Long[]>) this.getAttribute(PROCESS_DEFINITIONS);

        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type", e);
        }
    }
}
