/**
 * Copyright (c) 2016-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl.monitoring.container.transport.remote;

import java.io.IOException;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.ow2.petals.jmx.api.api.exception.AttributeErrorException;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.remote.IncomingConnectionObjectPoolMetrics;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.remote.OutgoingConnectionObjectPoolMetrics;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.remote.TcpTransporterMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.remote.exception.TcpTransporterMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.remote.exception.TcpTransporterMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.impl.AbstractServiceClient;

/**
 * Client API of the JMX service providing TCP transporter monitoring metrics.
 * 
 * @author Christophe Deneux - Linagora
 */
public final class TcpTransporterMonitoringServiceClientImpl extends AbstractServiceClient
        implements TcpTransporterMonitoringServiceClient {

    /**
     * Attribute : OutgoingMessagesCount
     */
    private static final String OUTGOING_MESSAGES_COUNT = "OutgoingMessagesCount";

    /**
     * Attribute : IncomingMessagesCount
     */
    private static final String INCOMING_MESSAGES_COUNT = "IncomingMessagesCount";

    /**
     * Attribute : EstablishedOutgoingConnections
     */
    private static final String ESTABLISHED_OUTGOING_CONNECTIONS = "EstablishedOutgoingConnections";

    /**
     * Attribute : OutgoingActiveConnectionsCurrent
     */
    private static final String ACTIVE_OUTGOING_CONNECTIONS_CURRENT = "OutgoingActiveConnectionsCurrent";

    /**
     * Attribute : OutgoingActiveConnectionsMax
     */
    private static final String ACTIVE_OUTGOING_CONNECTIONS_MAX = "OutgoingActiveConnectionsMax";

    /**
     * Attribute : OutgoingIdleConnectionsCurrent
     */
    private static final String IDLE_OUTGOING_CONNECTIONS_CURRENT = "OutgoingIdleConnectionsCurrent";

    /**
     * Attribute : OutgoingIdleConnectionsMax
     */
    private static final String IDLE_OUTGOING_CONNECTIONS_MAX = "OutgoingIdleConnectionsMax";

    /**
     * Attribute : OutgoingConnectionsMin
     */
    private static final String OUTGOING_CONNECTIONS_MIN = "OutgoingConnectionsMin";

    /**
     * Attribute : OutgoingConnectionsMax
     */
    private static final String OUTGOING_CONNECTIONS_MAX = "OutgoingConnectionsMax";

    /**
     * Attribute : EstablishedIncomingConnections
     */
    private static final String ESTABLISHED_INCOMING_CONNECTIONS = "EstablishedIncomingConnections";

    /**
     * Attribute : IncomingActiveConnectionsCurrent
     */
    private static final String ACTIVE_INCOMING_CONNECTIONS_CURRENT = "IncomingActiveConnectionsCurrent";

    /**
     * Attribute : IncomingActiveConnectionsMax
     */
    private static final String ACTIVE_INCOMING_CONNECTIONS_MAX = "IncomingActiveConnectionsMax";

    /**
     * Attribute : IncomingIdleConnectionsCurrent
     */
    private static final String IDLE_INCOMING_CONNECTIONS_CURRENT = "IncomingIdleConnectionsCurrent";

    /**
     * Attribute : IncomingIdleConnectionsMax
     */
    private static final String IDLE_INCOMING_CONNECTIONS_MAX = "IncomingIdleConnectionsMax";

    /**
     * Creates a new instance of {@link TcpTransporterMonitoringServiceClient}
     * 
     * @param petalsDomain
     * @param mBeanServerConnection
     * @throws TcpTransporterMonitoringServiceDoesNotExistException
     * @throws TcpTransporterMonitoringServiceErrorException
     */
    public TcpTransporterMonitoringServiceClientImpl(final String petalsDomain,
            final MBeanServerConnection mBeanServerConnection)
                    throws TcpTransporterMonitoringServiceDoesNotExistException,
                    TcpTransporterMonitoringServiceErrorException {

        super(petalsDomain, mBeanServerConnection);

        try {
            final Hashtable<String, String> attributes = new Hashtable<String, String>();
            attributes.put(JMX_FIELD_NAME, TcpTransporterMonitoringServiceClient.TCP_TRANSPORTER_MONITORING_MBEAN_NAME);
            attributes.put(JMX_FIELD_TYPE, JMX_SERVICE_TYPE);
            final ObjectName objName = new ObjectName(petalsDomain, attributes);
            final Set<ObjectName> objNames = this.mBeanServerConnection.queryNames(objName, null);
            if (objNames != null && objNames.size() == 1) {
                this.mbeanName = objNames.iterator().next();
            } else {
                throw new TcpTransporterMonitoringServiceDoesNotExistException();
            }
        } catch (final MalformedObjectNameException e) {
            throw new TcpTransporterMonitoringServiceErrorException(e);
        } catch (final IOException e) {
            throw new TcpTransporterMonitoringServiceErrorException(e);
        }
    }

    @Override
    public Map<String[], Long> getDeliveredOutgoingMessagesCount()
            throws TcpTransporterMonitoringServiceErrorException {

        try {
            return (Map<String[], Long>) this.getAttribute(OUTGOING_MESSAGES_COUNT);
        } catch (final AttributeErrorException e) {
            throw new TcpTransporterMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new TcpTransporterMonitoringServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public Map<String[], Long> getDeliveredIncomingMessagesCount()
            throws TcpTransporterMonitoringServiceErrorException {

        try {
            return (Map<String[], Long>) this.getAttribute(INCOMING_MESSAGES_COUNT);
        } catch (final AttributeErrorException e) {
            throw new TcpTransporterMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new TcpTransporterMonitoringServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public Map<String, OutgoingConnectionObjectPoolMetrics> getOutgoingConnectionPoolMetrics()
            throws TcpTransporterMonitoringServiceErrorException {
        try {
            final Map<String, OutgoingConnectionObjectPoolMetrics> connectionPoolMetrics = new HashMap<String, OutgoingConnectionObjectPoolMetrics>();

            // Current active connections
            final Map<String[], Long> activeConnectionsCurrent = (Map<String[], Long>) this
                    .getAttribute(ACTIVE_OUTGOING_CONNECTIONS_CURRENT);
            for (final Entry<String[], Long> entry : activeConnectionsCurrent.entrySet()) {
                OutgoingConnectionObjectPoolMetricsImpl connectionPoolMetric;
                if ((connectionPoolMetric = (OutgoingConnectionObjectPoolMetricsImpl) connectionPoolMetrics
                        .get(entry.getKey()[0])) == null) {
                    final OutgoingConnectionObjectPoolMetricsImpl newConnectionPoolMetric = new OutgoingConnectionObjectPoolMetricsImpl();
                    connectionPoolMetrics.put(entry.getKey()[0], newConnectionPoolMetric);
                    connectionPoolMetric = newConnectionPoolMetric;
                }
                connectionPoolMetric.setBorrowedObjectNumber(entry.getValue());
            }

            // Max active connections
            final Map<String[], Long> activeConnectionsMax = (Map<String[], Long>) this
                    .getAttribute(ACTIVE_OUTGOING_CONNECTIONS_MAX);
            for (final Entry<String[], Long> entry : activeConnectionsMax.entrySet()) {
                OutgoingConnectionObjectPoolMetricsImpl connectionPoolMetric;
                if ((connectionPoolMetric = (OutgoingConnectionObjectPoolMetricsImpl) connectionPoolMetrics
                        .get(entry.getKey()[0])) == null) {
                    final OutgoingConnectionObjectPoolMetricsImpl newConnectionPoolMetric = new OutgoingConnectionObjectPoolMetricsImpl();
                    connectionPoolMetrics.put(entry.getKey()[0], newConnectionPoolMetric);
                    connectionPoolMetric = newConnectionPoolMetric;
                }
                connectionPoolMetric.setBorrowedObjectMaxNumber(entry.getValue());
            }

            // Current idle connections
            final Map<String[], Long> idleConnectionsCurrent = (Map<String[], Long>) this
                    .getAttribute(IDLE_OUTGOING_CONNECTIONS_CURRENT);
            for (final Entry<String[], Long> entry : idleConnectionsCurrent.entrySet()) {
                OutgoingConnectionObjectPoolMetricsImpl connectionPoolMetric;
                if ((connectionPoolMetric = (OutgoingConnectionObjectPoolMetricsImpl) connectionPoolMetrics
                        .get(entry.getKey()[0])) == null) {
                    final OutgoingConnectionObjectPoolMetricsImpl newConnectionPoolMetric = new OutgoingConnectionObjectPoolMetricsImpl();
                    connectionPoolMetrics.put(entry.getKey()[0], newConnectionPoolMetric);
                    connectionPoolMetric = newConnectionPoolMetric;
                }
                connectionPoolMetric.setIdleObjectNumber(entry.getValue());
            }

            // Max idle connections
            final Map<String[], Long> idleConnectionsMax = (Map<String[], Long>) this
                    .getAttribute(IDLE_OUTGOING_CONNECTIONS_MAX);
            for (final Entry<String[], Long> entry : idleConnectionsMax.entrySet()) {
                OutgoingConnectionObjectPoolMetricsImpl connectionPoolMetric;
                if ((connectionPoolMetric = (OutgoingConnectionObjectPoolMetricsImpl) connectionPoolMetrics
                        .get(entry.getKey()[0])) == null) {
                    final OutgoingConnectionObjectPoolMetricsImpl newConnectionPoolMetric = new OutgoingConnectionObjectPoolMetricsImpl();
                    connectionPoolMetrics.put(entry.getKey()[0], newConnectionPoolMetric);
                    connectionPoolMetric = newConnectionPoolMetric;
                }
                connectionPoolMetric.setIdleObjectMaxNumber(entry.getValue());
            }

            // Min connections
            final Map<String[], Long> connectionsMin = (Map<String[], Long>) this
                    .getAttribute(OUTGOING_CONNECTIONS_MIN);
            for (final Entry<String[], Long> entry : connectionsMin.entrySet()) {
                OutgoingConnectionObjectPoolMetricsImpl connectionPoolMetric;
                if ((connectionPoolMetric = (OutgoingConnectionObjectPoolMetricsImpl) connectionPoolMetrics
                        .get(entry.getKey()[0])) == null) {
                    final OutgoingConnectionObjectPoolMetricsImpl newConnectionPoolMetric = new OutgoingConnectionObjectPoolMetricsImpl();
                    connectionPoolMetrics.put(entry.getKey()[0], newConnectionPoolMetric);
                    connectionPoolMetric = newConnectionPoolMetric;
                }
                connectionPoolMetric.setMinIdleSize(entry.getValue());
            }

            // Max connections
            final Map<String[], Long> connectionsMax = (Map<String[], Long>) this
                    .getAttribute(OUTGOING_CONNECTIONS_MAX);
            for (final Entry<String[], Long> entry : connectionsMax.entrySet()) {
                OutgoingConnectionObjectPoolMetricsImpl connectionPoolMetric;
                if ((connectionPoolMetric = (OutgoingConnectionObjectPoolMetricsImpl) connectionPoolMetrics
                        .get(entry.getKey()[0])) == null) {
                    final OutgoingConnectionObjectPoolMetricsImpl newConnectionPoolMetric = new OutgoingConnectionObjectPoolMetricsImpl();
                    connectionPoolMetrics.put(entry.getKey()[0], newConnectionPoolMetric);
                    connectionPoolMetric = newConnectionPoolMetric;
                }
                connectionPoolMetric.setMaxSize(entry.getValue());
            }

            // Established connections
            final Map<String[], Long> establishedConnections = (Map<String[], Long>) this
                    .getAttribute(ESTABLISHED_OUTGOING_CONNECTIONS);
            for (final Entry<String[], Long> entry : establishedConnections.entrySet()) {
                OutgoingConnectionObjectPoolMetricsImpl connectionPoolMetric;
                if ((connectionPoolMetric = (OutgoingConnectionObjectPoolMetricsImpl) connectionPoolMetrics
                        .get(entry.getKey()[0])) == null) {
                    final OutgoingConnectionObjectPoolMetricsImpl newConnectionPoolMetric = new OutgoingConnectionObjectPoolMetricsImpl();
                    connectionPoolMetrics.put(entry.getKey()[0], newConnectionPoolMetric);
                    connectionPoolMetric = newConnectionPoolMetric;
                }
                connectionPoolMetric.setEstablished(entry.getValue());
            }

            return connectionPoolMetrics;
        } catch (final AttributeErrorException e) {
            throw new TcpTransporterMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new TcpTransporterMonitoringServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public Map<String, IncomingConnectionObjectPoolMetrics> getIncomingConnectionPoolMetrics()
            throws TcpTransporterMonitoringServiceErrorException {
        try {
            final Map<String, IncomingConnectionObjectPoolMetrics> connectionMetrics = new HashMap<String, IncomingConnectionObjectPoolMetrics>();

            // Current active connections
            final Map<String[], Long> activeConnectionsCurrent = (Map<String[], Long>) this
                    .getAttribute(ACTIVE_INCOMING_CONNECTIONS_CURRENT);
            for (final Entry<String[], Long> entry : activeConnectionsCurrent.entrySet()) {
                IncomingConnectionObjectPoolMetricsImpl connectionMetric;
                if ((connectionMetric = (IncomingConnectionObjectPoolMetricsImpl) connectionMetrics
                        .get(entry.getKey()[0])) == null) {
                    final IncomingConnectionObjectPoolMetricsImpl newConnectionMetric = new IncomingConnectionObjectPoolMetricsImpl();
                    connectionMetrics.put(entry.getKey()[0], newConnectionMetric);
                    connectionMetric = newConnectionMetric;
                }
                connectionMetric.setBorrowedObjectNumber(entry.getValue());
            }

            // Max active connections
            final Map<String[], Long> activeConnectionsMax = (Map<String[], Long>) this
                    .getAttribute(ACTIVE_INCOMING_CONNECTIONS_MAX);
            for (final Entry<String[], Long> entry : activeConnectionsMax.entrySet()) {
                IncomingConnectionObjectPoolMetricsImpl connectionMetric;
                if ((connectionMetric = (IncomingConnectionObjectPoolMetricsImpl) connectionMetrics
                        .get(entry.getKey()[0])) == null) {
                    final IncomingConnectionObjectPoolMetricsImpl newConnectionMetric = new IncomingConnectionObjectPoolMetricsImpl();
                    connectionMetrics.put(entry.getKey()[0], newConnectionMetric);
                    connectionMetric = newConnectionMetric;
                }
                connectionMetric.setBorrowedObjectMaxNumber(entry.getValue());
            }

            // Current idle connections
            final Map<String[], Long> idleConnectionsCurrent = (Map<String[], Long>) this
                    .getAttribute(IDLE_INCOMING_CONNECTIONS_CURRENT);
            for (final Entry<String[], Long> entry : idleConnectionsCurrent.entrySet()) {
                IncomingConnectionObjectPoolMetricsImpl connectionMetric;
                if ((connectionMetric = (IncomingConnectionObjectPoolMetricsImpl) connectionMetrics
                        .get(entry.getKey()[0])) == null) {
                    final IncomingConnectionObjectPoolMetricsImpl newConnectionMetric = new IncomingConnectionObjectPoolMetricsImpl();
                    connectionMetrics.put(entry.getKey()[0], newConnectionMetric);
                    connectionMetric = newConnectionMetric;
                }
                connectionMetric.setIdleObjectNumber(entry.getValue());
            }

            // Max idle connections
            final Map<String[], Long> idleConnectionsMax = (Map<String[], Long>) this
                    .getAttribute(IDLE_INCOMING_CONNECTIONS_MAX);
            for (final Entry<String[], Long> entry : idleConnectionsMax.entrySet()) {
                IncomingConnectionObjectPoolMetricsImpl connectionMetric;
                if ((connectionMetric = (IncomingConnectionObjectPoolMetricsImpl) connectionMetrics
                        .get(entry.getKey()[0])) == null) {
                    final IncomingConnectionObjectPoolMetricsImpl newConnectionMetric = new IncomingConnectionObjectPoolMetricsImpl();
                    connectionMetrics.put(entry.getKey()[0], newConnectionMetric);
                    connectionMetric = newConnectionMetric;
                }
                connectionMetric.setIdleObjectMaxNumber(entry.getValue());
            }

            // Established connections
            final Map<String[], Long> establishedConnections = (Map<String[], Long>) this
                    .getAttribute(ESTABLISHED_INCOMING_CONNECTIONS);
            for (final Entry<String[], Long> entry : establishedConnections.entrySet()) {
                IncomingConnectionObjectPoolMetricsImpl connectionMetric;
                if ((connectionMetric = (IncomingConnectionObjectPoolMetricsImpl) connectionMetrics
                        .get(entry.getKey()[0])) == null) {
                    final IncomingConnectionObjectPoolMetricsImpl newConnectionMetric = new IncomingConnectionObjectPoolMetricsImpl();
                    connectionMetrics.put(entry.getKey()[0], newConnectionMetric);
                    connectionMetric = newConnectionMetric;
                }
                connectionMetric.setEstablished(entry.getValue());
            }

            return connectionMetrics;
        } catch (final AttributeErrorException e) {
            throw new TcpTransporterMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new TcpTransporterMonitoringServiceErrorException("Unexpected result type", e);
        }
    }
}
