/**
 * Copyright (c) 2014-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl;

import javax.jbi.messaging.MessageExchange;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.ObjectName;

import org.easymock.EasyMock;
import org.junit.Before;
import org.junit.Test;
import org.ow2.petals.jmx.api.api.AdminServiceClient;
import org.ow2.petals.jmx.api.api.exception.AdminServiceErrorException;
import org.ow2.petals.jmx.api.impl.mbean.AdminService;

/**
 * <p>
 * Unit tests of {@link AdminServiceClientImpl} to check that JMX client interfaces are aligned with the JMX server
 * MBean part.
 * </p>
 * <p>
 * This unit tests implement {@link AdminServiceClient} to avoid to forget some methods. A unit test will only invoke
 * the associated method of {@link AdminServiceClient}
 * </p>
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class AdminServiceClientImplTest extends AbstractServiceClientImpl implements AdminServiceClient {

    private AdminServiceClient deploymentServiceClient;

    @Before
    public void before() throws Exception {
        this.embeddedJmxSrvCon.registerAdminService(new AdminService());
        this.deploymentServiceClient = jmxClient.getAdminServiceClient();
    }

    /**
     * Check the interface consistency of {@link AdminServiceClientImpl#getSystemInfo()}
     */
    @Test
    public void testGetSystemInfo() throws AdminServiceErrorException {

        this.getSystemInfo();
    }

    @Override
    public String getSystemInfo() throws AdminServiceErrorException {

        return this.deploymentServiceClient.getSystemInfo();
    }

    /**
     * Check the interface consistency of {@link AdminServiceClientImpl#getBindingComponents()}
     */
    @Test
    public void testGetBindingComponents() throws AdminServiceErrorException {

        this.getBindingComponents();
    }

    @Override
    public ObjectName[] getBindingComponents() throws AdminServiceErrorException {

        return this.deploymentServiceClient.getBindingComponents();
    }

    /**
     * Check the interface consistency of {@link AdminServiceClientImpl#getEngineComponents()}
     */
    @Test
    public void testGetEngineComponents() throws AdminServiceErrorException {

        this.getEngineComponents();
    }

    @Override
    public ObjectName[] getEngineComponents() throws AdminServiceErrorException {

        return this.deploymentServiceClient.getEngineComponents();
    }

    /**
     * Check the interface consistency of {@link AdminServiceClientImpl#getComponentByName(String)}
     */
    @Test
    public void testGetComponentByName() throws AdminServiceErrorException {

        this.getComponentByName("component");
    }

    @Override
    public ObjectName getComponentByName(final String component) throws AdminServiceErrorException {

        return this.deploymentServiceClient.getComponentByName(component);
    }

    /**
     * Check the interface consistency of {@link AdminServiceClientImpl#isBinding(String)}
     */
    @Test
    public void testIsBinding() throws AdminServiceErrorException {

        this.isBinding("componentName");
    }

    @Override
    public boolean isBinding(final String componentName) throws AdminServiceErrorException {

        return this.deploymentServiceClient.isBinding(componentName);
    }

    /**
     * Check the interface consistency of {@link AdminServiceClientImpl#isEngine(String)}
     */
    @Test
    public void testIsEngine() throws AdminServiceErrorException {

        this.isEngine("componentName");
    }

    @Override
    public boolean isEngine(final String componentName) throws AdminServiceErrorException {

        return this.deploymentServiceClient.isEngine(componentName);
    }

    /**
     * Check the interface consistency of
     * {@link AdminServiceClientImpl#isExchangeWithConsumerOkayForComponent(String, ServiceEndpoint, MessageExchange)}
     */
    @Test
    public void testIsExchangeWithConsumerOkayForComponent() throws AdminServiceErrorException {

        final ServiceEndpoint serviceEndpointMock = EasyMock.createMock(ServiceEndpoint.class);
        final MessageExchange messageExchangeMock = EasyMock.createMock(MessageExchange.class);
        EasyMock.replay(serviceEndpointMock, messageExchangeMock);

        this.isExchangeWithConsumerOkayForComponent("containerName", serviceEndpointMock, messageExchangeMock);
    }

    @Override
    public boolean isExchangeWithConsumerOkayForComponent(final String containerName,
            final ServiceEndpoint serviceEndpoint, final MessageExchange exchange) throws AdminServiceErrorException {

        return this.deploymentServiceClient.isExchangeWithConsumerOkayForComponent(containerName, serviceEndpoint,
                exchange);
    }

    /**
     * Check the interface consistency of
     * {@link AdminServiceClientImpl#isExchangeWithProviderOkayForComponent(String, ServiceEndpoint, MessageExchange)}
     */
    @Test
    public void testIsExchangeWithProviderOkayForComponent() throws AdminServiceErrorException {

        final ServiceEndpoint serviceEndpointMock = EasyMock.createMock(ServiceEndpoint.class);
        final MessageExchange messageExchangeMock = EasyMock.createMock(MessageExchange.class);
        EasyMock.replay(serviceEndpointMock, messageExchangeMock);

        this.isExchangeWithProviderOkayForComponent("containerName", serviceEndpointMock, messageExchangeMock);
    }

    @Override
    public boolean isExchangeWithProviderOkayForComponent(final String containerName,
            final ServiceEndpoint serviceEndpoint, final MessageExchange exchange) throws AdminServiceErrorException {

        return this.deploymentServiceClient.isExchangeWithProviderOkayForComponent(containerName, serviceEndpoint,
                exchange);
    }

}
