/**
 * Copyright (c) 2014-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl;

import org.junit.Before;
import org.junit.Test;
import org.ow2.petals.jmx.api.api.ComponentClient;
import org.ow2.petals.jmx.api.api.configuration.component.RuntimeConfigurationComponentClient;
import org.ow2.petals.jmx.api.api.configuration.component.exception.ConfigurationComponentDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.ComponentDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.ComponentErrorException;
import org.ow2.petals.jmx.api.api.exception.ComponentUnsupportedOperationException;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.impl.mbean.ComponentLifeCycle;
import org.ow2.petals.jmx.api.impl.mbean.configuration.component.RuntimeConfigurationComponentService;

/**
 * <p>
 * Unit tests of {@link ComponentClientImpl} to check that JMX client interfaces are aligned with the JMX server MBean
 * part.
 * </p>
 * <p>
 * This unit tests implement {@link ComponentClient} to avoid to forget some methods. A unit test will only invoke the
 * associated method of {@link ComponentClient}
 * </p>
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class ComponentClientImplTest extends AbstractServiceClientImpl implements ComponentClient {

    private static final String COMPONENT_ID_NOTSUPPORTING_RELOADING = "component-not-supporting-reloading";

    private ComponentClient componentServiceClient;

    @Before
    public void before() throws Exception {
        this.embeddedJmxSrvCon.registerComponentLifeCycleService(new ComponentLifeCycle(), COMPONENT_ID)
                .registerRuntimeConfigurationComponentService(new RuntimeConfigurationComponentService(), COMPONENT_ID)
                .registerComponentLifeCycleService(new ComponentLifeCycle(), COMPONENT_ID_NOTSUPPORTING_RELOADING);
        this.componentServiceClient = this.jmxClient.getComponentClient(COMPONENT_ID);
    }

    /**
     * Check the interface consistency of {@link ComponentClientImpl#start()}
     */
    @Test
    public void testStart() throws ComponentErrorException {

        this.start();
    }

    @Override
    public void start() throws ComponentErrorException {

        this.componentServiceClient.start();
    }

    /**
     * Check the interface consistency of {@link ComponentClientImpl#stop()}
     */
    @Test
    public void testStop() throws ComponentErrorException {

        this.stop();
    }

    @Override
    public void stop() throws ComponentErrorException {

        this.componentServiceClient.stop();
    }

    /**
     * Check the interface consistency of {@link ComponentClientImpl#shutdown()}
     */
    @Test
    public void testShutdown() throws ComponentErrorException {

        this.shutdown();
    }

    @Override
    public void shutdown() throws ComponentErrorException {

        this.componentServiceClient.shutdown();
    }

    /**
     * Check the interface consistency of {@link ComponentClientImpl#getCurrentState()}
     */
    @Test
    public void testGetCurrentState() throws ComponentErrorException {

        this.getCurrentState();
    }

    @Override
    public State getCurrentState() throws ComponentErrorException {

        return this.componentServiceClient.getCurrentState();
    }

    /**
     * Check the interface consistency of {@link ComponentClientImpl#reloadConfiguration()}
     */
    @Test
    public void testReloadConfiguration() throws ComponentErrorException {

        this.reloadConfiguration();
    }

    @Override
    public void reloadConfiguration() throws ComponentUnsupportedOperationException, ComponentErrorException {

        this.componentServiceClient.reloadConfiguration();
    }

    /**
     * Check the interface consistency of {@link ComponentClientImpl#reloadConfiguration()}
     */
    @Test(expected = ComponentUnsupportedOperationException.class)
    public void testReloadConfiguration_ComponentNotSupportReload() throws ComponentErrorException,
            ComponentDoesNotExistException, ConnectionErrorException {

        this.jmxClient.getComponentClient(COMPONENT_ID_NOTSUPPORTING_RELOADING).reloadConfiguration();
    }

    @Override
    public RuntimeConfigurationComponentClient getRuntimeConfigurationClient()
            throws ConfigurationComponentDoesNotExistException {
        return this.componentServiceClient.getRuntimeConfigurationClient();
    }

    @Override
    public <T> T getRuntimeConfigurationClient(Class<T> clazz) throws ConfigurationComponentDoesNotExistException {
        return this.componentServiceClient.getRuntimeConfigurationClient(clazz);
    }

}
