/**
 * Copyright (c) 2014-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl;

import java.net.MalformedURLException;
import java.net.URL;

import org.junit.Before;
import org.junit.Test;
import org.ow2.petals.jmx.api.api.DeploymentServiceClient;
import org.ow2.petals.jmx.api.api.exception.DeploymentServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.ServiceAssemblyDoesNotExistException;
import org.ow2.petals.jmx.api.impl.mbean.DeploymentService;

/**
 * <p>
 * Unit tests of {@link DeploymentServiceClientImpl} to check that JMX client interfaces are aligned with the JMX server
 * MBean part.
 * </p>
 * <p>
 * This unit tests implement {@link DeploymentServiceClient} to avoid to forget some methods. A unit test will only
 * invoke the associated method of {@link DeploymentServiceClient}
 * </p>
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class DeploymentServiceClientImplTest extends AbstractServiceClientImpl implements DeploymentServiceClient {

    private DeploymentServiceClient deploymentServiceClient;

    @Before
    public void before() throws Exception {
        this.embeddedJmxSrvCon.registerDeploymentService(new DeploymentService());
        this.deploymentServiceClient = jmxClient.getDeploymentServiceClient();
    }

    /**
     * Check the interface consistency of {@link DeploymentServiceClientImpl#deploy(URL)}
     */
    @Test
    public void testDeploy() throws DeploymentServiceErrorException, MalformedURLException {

        this.deploy(new URL("http://localhost"));

    }

    @Override
    public String deploy(final URL packageURL) throws DeploymentServiceErrorException {

        return this.deploymentServiceClient.deploy(packageURL);
    }

    /**
     * Check the interface consistency of {@link DeploymentServiceClientImpl#forceUndeploy(String)}
     */
    @Test
    public void testForceUndeploy() throws DeploymentServiceErrorException {

        this.forceUndeploy("serviceAssemblyName");
    }

    @Override
    public boolean forceUndeploy(final String serviceAssemblyName) throws DeploymentServiceErrorException {
        return this.deploymentServiceClient.forceUndeploy(serviceAssemblyName);
    }

    /**
     * Check the interface consistency of {@link DeploymentServiceClientImpl#getDeployedServiceAssemblies()}
     */
    @Test
    public void testGetDeployedServiceAssemblies() throws DeploymentServiceErrorException {

        this.getDeployedServiceAssemblies();
    }

    @Override
    public String[] getDeployedServiceAssemblies() throws DeploymentServiceErrorException {

        return this.deploymentServiceClient.getDeployedServiceAssemblies();
    }

    /**
     * Check the interface consistency of
     * {@link DeploymentServiceClientImpl#getDeployedServiceAssembliesForComponent(String)}
     */
    @Test
    public void testGetDeployedServiceAssembliesForComponent() throws DeploymentServiceErrorException {

        this.getDeployedServiceAssembliesForComponent("componentName");
    }

    @Override
    public String[] getDeployedServiceAssembliesForComponent(final String componentName)
            throws DeploymentServiceErrorException {

        return this.deploymentServiceClient.getDeployedServiceAssembliesForComponent(componentName);
    }

    /**
     * Check the interface consistency of {@link DeploymentServiceClientImpl#getServiceAssemblyDescriptor(String)}
     */
    @Test
    public void testGetServiceAssemblyDescriptor() throws DeploymentServiceErrorException {

        this.getServiceAssemblyDescriptor("serviceAssemblyName");
    }

    @Override
    public String getServiceAssemblyDescriptor(final String serviceAssemblyName) throws DeploymentServiceErrorException {

        return this.deploymentServiceClient.getServiceAssemblyDescriptor(serviceAssemblyName);
    }

    /**
     * Check the interface consistency of {@link DeploymentServiceClientImpl#getServiceUnitForServiceAssembly(String)}
     */
    @Test
    public void testGetServiceUnitForServiceAssembly() throws DeploymentServiceErrorException {

        this.getServiceUnitForServiceAssembly("serviceAssemblyName");
    }

    @Override
    public String[] getServiceUnitForServiceAssembly(final String serviceAssemblyName)
            throws DeploymentServiceErrorException {

        return this.deploymentServiceClient.getServiceUnitForServiceAssembly(serviceAssemblyName);
    }

    /**
     * Check the interface consistency of {@link DeploymentServiceClientImpl#getState(String)}
     */
    @Test
    public void testGetState() throws DeploymentServiceErrorException, ServiceAssemblyDoesNotExistException {

        this.getState("serviceAssemblyName");
    }

    @Override
    public State getState(final String serviceAssemblyName) throws ServiceAssemblyDoesNotExistException,
            DeploymentServiceErrorException {

        return this.deploymentServiceClient.getState(serviceAssemblyName);
    }

    /**
     * Check the interface consistency of {@link DeploymentServiceClientImpl#isServiceAssemblyDeployed(String)}
     */
    @Test
    public void testIsServiceAssemblyDeployed() throws DeploymentServiceErrorException {

        this.isServiceAssemblyDeployed("serviceAssembly");
    }

    @Override
    public boolean isServiceAssemblyDeployed(final String serviceAssembly) throws DeploymentServiceErrorException {

        return this.deploymentServiceClient.isServiceAssemblyDeployed(serviceAssembly);
    }

    /**
     * Check the interface consistency of {@link DeploymentServiceClientImpl#shutdown(String)}
     */
    @Test
    public void testShutdown() throws DeploymentServiceErrorException {

        this.shutdown("serviceAssemblyName");
    }

    @Override
    public String shutdown(final String serviceAssemblyName) throws DeploymentServiceErrorException {

        return this.deploymentServiceClient.shutdown(serviceAssemblyName);
    }

    /**
     * Check the interface consistency of {@link DeploymentServiceClientImpl#shutdownAllServiceAssemblies()}
     */
    @Test
    public void testShutdownAllServiceAssemblies() throws DeploymentServiceErrorException {

        this.shutdownAllServiceAssemblies();
    }

    @Override
    public String[] shutdownAllServiceAssemblies() throws DeploymentServiceErrorException {

        return this.deploymentServiceClient.shutdownAllServiceAssemblies();
    }

    /**
     * Check the interface consistency of {@link DeploymentServiceClientImpl#start(String)}
     */
    @Test
    public void testStart() throws DeploymentServiceErrorException {

        this.start("serviceAssemblyName");
    }

    @Override
    public String start(final String serviceAssemblyName) throws DeploymentServiceErrorException {

        return this.deploymentServiceClient.start(serviceAssemblyName);
    }

    /**
     * Check the interface consistency of {@link DeploymentServiceClientImpl#startAllServiceAssemblies()}
     */
    @Test
    public void testStartAllServiceAssemblies() throws DeploymentServiceErrorException {

        this.startAllServiceAssemblies();
    }

    @Override
    public String[] startAllServiceAssemblies() throws DeploymentServiceErrorException {

        return this.deploymentServiceClient.startAllServiceAssemblies();
    }

    /**
     * Check the interface consistency of {@link DeploymentServiceClientImpl#stop(String)}
     */
    @Test
    public void testStop() throws DeploymentServiceErrorException {

        this.stop("serviceAssemblyName");
    }

    @Override
    public String stop(final String serviceAssemblyName) throws DeploymentServiceErrorException {

        return this.deploymentServiceClient.stop(serviceAssemblyName);
    }

    /**
     * Check the interface consistency of {@link DeploymentServiceClientImpl#stopAllServiceAssemblies()}
     */
    @Test
    public void testStopAllServiceAssemblies() throws DeploymentServiceErrorException {

        this.stopAllServiceAssemblies();
    }

    @Override
    public String[] stopAllServiceAssemblies() throws DeploymentServiceErrorException {

        return this.deploymentServiceClient.stopAllServiceAssemblies();
    }

    /**
     * Check the interface consistency of {@link DeploymentServiceClientImpl#undeploy(String)}
     */
    @Test
    public void testUndeploy() throws DeploymentServiceErrorException {

        this.undeploy("serviceAssemblyName");
    }

    @Override
    public String undeploy(final String serviceAssemblyName) throws DeploymentServiceErrorException {

        return this.deploymentServiceClient.undeploy(serviceAssemblyName);
    }

    /**
     * Check the interface consistency of {@link DeploymentServiceClientImpl#undeployAllServiceAssemblies(boolean)}
     */
    @Test
    public void testUndeployAllServiceAssemblies() throws DeploymentServiceErrorException {

        this.undeployAllServiceAssemblies(false);
    }

    @Override
    public String[] undeployAllServiceAssemblies(final boolean force) throws DeploymentServiceErrorException {

        return this.deploymentServiceClient.undeployAllServiceAssemblies(force);
    }

}
