/**
 * Copyright (c) 2014-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl;

import java.net.InetAddress;

import javax.management.ObjectName;
import javax.management.remote.JMXConnectorServer;

import org.junit.rules.ExternalResource;
import org.ow2.petals.jmx.api.api.AdminServiceClient;
import org.ow2.petals.jmx.api.api.ComponentClient;
import org.ow2.petals.jmx.api.api.DeploymentServiceClient;
import org.ow2.petals.jmx.api.api.EndpointDirectoryClient;
import org.ow2.petals.jmx.api.api.InstallationServiceClient;
import org.ow2.petals.jmx.api.api.InstallerComponentClient;
import org.ow2.petals.jmx.api.api.JMXClient;
import org.ow2.petals.jmx.api.api.LoggerServiceClient;
import org.ow2.petals.jmx.api.api.PetalsAdminServiceClient;
import org.ow2.petals.jmx.api.api.SystemMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.TopologyServiceClient;
import org.ow2.petals.jmx.api.api.exception.AdminDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.AdminServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.ComponentDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.ComponentErrorException;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.exception.DeploymentServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.DeploymentServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.EndpointDirectoryServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.EndpointDirectoryServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.InstallationServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.InstallationServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.InstallerComponentDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.LoggerServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.LoggerServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.NotRemoteJMXClientException;
import org.ow2.petals.jmx.api.api.exception.PetalsAdminDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.PetalsAdminServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.SystemMonitoringDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.SystemMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.TopologyServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.TopologyServiceErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.ComponentMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.se.flowable.SeFlowableMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.monitoring.component.soap.PetalsBcSoapMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.local.LocalTransporterMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.local.exception.LocalTransporterMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.local.exception.LocalTransporterMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.remote.TcpTransporterMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.remote.exception.TcpTransporterMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.remote.exception.TcpTransporterMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.api.monitoring.defect.DefectFilter;
import org.ow2.petals.jmx.api.api.monitoring.defect.DefectListener;
import org.ow2.petals.jmx.api.api.monitoring.defect.exception.DefectEmitterDoesNotExistsException;
import org.ow2.petals.jmx.api.api.monitoring.defect.exception.DefectListenerException;
import org.ow2.petals.jmx.api.api.monitoring.defect.exception.NoDefectListenerException;

/**
 * The EmbeddedJmxClient Rule allows creation of an embedded Petals JMX client that are guaranteed to be deleted when
 * the test method finishes (whether it passes or fails):
 * 
 * <pre>
 * public static class ... {
 *  &#064;Rule
 *  public EmbeddedJmxClient jmxClient = new EmbeddedJmxClient(port, credentials);
 * 
 *  &#064;Test
 *  public void test...() throws IOException {
 *      jmxClient.getTopologyServiceClient(...);
 *      ...
 *  }
 * }
 * </pre>
 */
public class EmbeddedJmxClient extends ExternalResource implements JMXClient {

    /**
     * The {@link JMXConnectorServer} listen on the localhost
     */
    private final static String JMX_HOST = "localhost";

    /**
     * A user having a non-empty password
     */
    private final static String JMX_USER = "petals";

    private final static String JMX_PASSWORD = "petals";

    /**
     * The listening port of the {@link JMXConnectorServer}
     */
    private final int jmxPort;

    private final String jmxHost;

    private final String jmxUsername;

    private final String jmxPassword;

    private JMXClient jmxClient;

    /**
     * Creates an EmbeddedJmxClient
     * 
     * @param hostname
     *            Hostname of the JMX server
     * @param jmxPort
     *            Port on which the JMX server listen
     * @param username
     *            Credential username required by the JMX server
     * @param password
     *            Credential password required by the JMX server
     */
    public EmbeddedJmxClient(final String hostname, final int jmxPort, final String username, final String password) {
        this.jmxHost = hostname;
        this.jmxPort = jmxPort;
        this.jmxUsername = username;
        this.jmxPassword = password;
    }

    /**
     * Creates an EmbeddedJmxClient with default credentials:
     * <ul>
     * <li>username: petals</li>
     * <li>password:petals</li>
     * </ul>
     * 
     * @param hostname
     *            Hostname of the JMX server
     * @param jmxPort
     *            The listening port of the {@link JMXConnectorServer}
     */
    public EmbeddedJmxClient(final String hostname, final int jmxPort) {
        this(hostname, jmxPort, JMX_USER, JMX_PASSWORD);
    }

    /**
     * Creates an EmbeddedJmxClient with default values except for credentials:
     * 
     * @param hostname
     *            Hostname of the JMX server
     * @param jmxPort
     *            Port on which the JMX server listen
     */
    public EmbeddedJmxClient(final String username, final String password) {
        this(JMX_HOST, JMXClient.DEFAULT_PORT, username, password);
    }

    /**
     * Creates an EmbeddedJmxClient with default values except for the JMX port:
     * <ul>
     * <li>host: localhost</li>
     * <li>username: petals</li>
     * <li>password:petals</li>
     * </ul>
     * 
     * @param jmxPort
     *            The listening port of the {@link JMXConnectorServer}
     */
    public EmbeddedJmxClient(final int jmxPort) {
        this(JMX_HOST, jmxPort, JMX_USER, JMX_PASSWORD);
    }

    /**
     * Creates an EmbeddedJmxClient with default values:
     * <ul>
     * <li>host: localhost</li>
     * <li>port: {@link JMXClient.DEFAULT_PORT}</li>
     * <li>username: petals</li>
     * <li>password:petals</li>
     * </ul>
     */
    public EmbeddedJmxClient() {
        this(JMX_HOST, JMXClient.DEFAULT_PORT, JMX_USER, JMX_PASSWORD);
    }

    @Override
    protected void before() throws Throwable {
        this.createAndConnectJmxClient();
    }

    @Override
    protected void after() {
        try {
            this.disconnectAndDestroyJmxClient();
        } catch (final Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Create and start a {@link JMXConnectorServer}
     */
    private void createAndConnectJmxClient() throws Exception {

        this.jmxClient = PetalsJmxApiFactory.getInstance().createJMXClient(this.jmxHost, this.jmxPort,
                this.jmxUsername, this.jmxPassword);
    }

    /**
     * Stop and destroy the {@link JMXConnectorServer}
     */
    private void disconnectAndDestroyJmxClient() throws Exception {
        this.jmxClient.disconnect();
        org.ow2.petals.jmx.api.api.PetalsJmxApiFactory.close();
    }

    @Override
    public InetAddress getHost() throws NotRemoteJMXClientException {
        return this.jmxClient.getHost();
    }

    @Override
    public int getPort() throws NotRemoteJMXClientException {
        return this.jmxClient.getPort();
    }

    @Override
    public String getUsername() throws NotRemoteJMXClientException {
        return this.jmxClient.getUsername();
    }

    @Override
    public String getPassword() throws NotRemoteJMXClientException {
        return this.jmxClient.getPassword();
    }

    @Override
    public PetalsAdminServiceClient getPetalsAdminServiceClient() throws PetalsAdminDoesNotExistException,
            PetalsAdminServiceErrorException, ConnectionErrorException {
        return this.jmxClient.getPetalsAdminServiceClient();
    }

    @Override
    public AdminServiceClient getAdminServiceClient() throws AdminDoesNotExistException, AdminServiceErrorException,
            ConnectionErrorException {
        return this.jmxClient.getAdminServiceClient();
    }

    @Override
    public InstallationServiceClient getInstallationServiceClient() throws InstallationServiceDoesNotExistException,
            InstallationServiceErrorException, ConnectionErrorException {
        return this.jmxClient.getInstallationServiceClient();
    }

    @Override
    public DeploymentServiceClient getDeploymentServiceClient() throws DeploymentServiceDoesNotExistException,
            DeploymentServiceErrorException, ConnectionErrorException {
        return this.jmxClient.getDeploymentServiceClient();
    }

    @Override
    public ComponentClient getComponentClient(final String componentName) throws ComponentDoesNotExistException,
            ComponentErrorException, ConnectionErrorException {
        return this.jmxClient.getComponentClient(componentName);
    }

    @Override
    public ComponentClient getComponentClient(final ObjectName componentMBeanName)
            throws ComponentDoesNotExistException, ComponentErrorException, ConnectionErrorException {
        return this.jmxClient.getComponentClient(componentMBeanName);
    }

    @Override
    public InstallerComponentClient getInstallerComponentClient(final String componentName)
            throws InstallerComponentDoesNotExistException, ComponentErrorException, ConnectionErrorException {
        return this.jmxClient.getInstallerComponentClient(componentName);
    }

    @Override
    public EndpointDirectoryClient getEndpointDirectoryClient() throws ConnectionErrorException,
            EndpointDirectoryServiceDoesNotExistException, EndpointDirectoryServiceErrorException {
        return this.jmxClient.getEndpointDirectoryClient();
    }

    @Override
    public SystemMonitoringServiceClient getSystemMonitoringServiceClient() throws ConnectionErrorException,
            SystemMonitoringDoesNotExistException, SystemMonitoringServiceErrorException {
        return this.jmxClient.getSystemMonitoringServiceClient();
    }

    @Override
    public LoggerServiceClient getLoggerServiceClient() throws LoggerServiceErrorException,
            LoggerServiceDoesNotExistException, ConnectionErrorException {
        return this.jmxClient.getLoggerServiceClient();
    }

    @Override
    public LocalTransporterMonitoringServiceClient getLocalTransporterMonitoringServiceClient()
            throws LocalTransporterMonitoringServiceErrorException,
            LocalTransporterMonitoringServiceDoesNotExistException, ConnectionErrorException {
        return this.jmxClient.getLocalTransporterMonitoringServiceClient();
    }

    @Override
    public TcpTransporterMonitoringServiceClient getTcpTransporterMonitoringServiceClient()
            throws TcpTransporterMonitoringServiceErrorException, TcpTransporterMonitoringServiceDoesNotExistException,
            ConnectionErrorException {
        return this.jmxClient.getTcpTransporterMonitoringServiceClient();
    }

    @Override
    public ComponentMonitoringServiceClient getComponentMonitoringServiceClient(final String componentIdentifier)
            throws ComponentMonitoringServiceErrorException, ComponentMonitoringServiceDoesNotExistException,
            ConnectionErrorException {
        return this.jmxClient.getComponentMonitoringServiceClient(componentIdentifier);
    }

    @Override
    public PetalsBcSoapMonitoringServiceClient getPetalsBcSoapMonitoringServiceClient(final String componentIdentifier)
            throws ComponentMonitoringServiceErrorException, ComponentMonitoringServiceDoesNotExistException,
            ConnectionErrorException {
        return this.jmxClient.getPetalsBcSoapMonitoringServiceClient(componentIdentifier);
    }

    @Override
    public SeFlowableMonitoringServiceClient getSeFlowableMonitoringServiceClient(final String componentIdentifier)
            throws ComponentMonitoringServiceErrorException, ComponentMonitoringServiceDoesNotExistException,
            ConnectionErrorException {
        return this.jmxClient.getSeFlowableMonitoringServiceClient(componentIdentifier);
    }

    @Override
    public void subscribeToComponentMonitoringService(final String componentIdentifier,
            final DefectListener defectListener, final DefectFilter defectFilter) throws DefectListenerException,
            DefectEmitterDoesNotExistsException {
        this.jmxClient.subscribeToComponentMonitoringService(componentIdentifier, defectListener, defectFilter);
    }

    @Override
    public void unsubscribeToComponentMonitoringService(final String componentIdentifier)
            throws NoDefectListenerException, DefectEmitterDoesNotExistsException, DefectListenerException {
        this.jmxClient.unsubscribeToComponentMonitoringService(componentIdentifier);
    }

    @Override
    public TopologyServiceClient getTopologyServiceClient() throws TopologyServiceErrorException,
            TopologyServiceDoesNotExistException, ConnectionErrorException {
        return this.jmxClient.getTopologyServiceClient();
    }

    @Override
    public void disconnect() throws ConnectionErrorException {
        this.jmxClient.disconnect();
    }
}
