/**
 * Copyright (c) 2014-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl;

import java.io.File;
import java.io.FileOutputStream;
import java.lang.management.ManagementFactory;
import java.lang.reflect.Method;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Properties;
import java.util.Vector;

import javax.jbi.management.ComponentLifeCycleMBean;
import javax.jbi.management.InstallerMBean;
import javax.management.Descriptor;
import javax.management.DynamicMBean;
import javax.management.InstanceAlreadyExistsException;
import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanRegistrationException;
import javax.management.MBeanServer;
import javax.management.MalformedObjectNameException;
import javax.management.NotCompliantMBeanException;
import javax.management.ObjectName;
import javax.management.RuntimeOperationsException;
import javax.management.modelmbean.DescriptorSupport;
import javax.management.modelmbean.InvalidTargetObjectTypeException;
import javax.management.modelmbean.ModelMBeanConstructorInfo;
import javax.management.modelmbean.ModelMBeanInfo;
import javax.management.modelmbean.ModelMBeanInfoSupport;
import javax.management.modelmbean.ModelMBeanNotificationInfo;
import javax.management.modelmbean.ModelMBeanOperationInfo;
import javax.management.modelmbean.RequiredModelMBean;
import javax.management.remote.JMXConnectorServer;
import javax.management.remote.JMXConnectorServerFactory;
import javax.management.remote.JMXServiceURL;

import org.junit.rules.ExternalResource;
import org.junit.rules.TemporaryFolder;
import org.ow2.petals.clientserverapi.admin.PetalsAdminService;
import org.ow2.petals.clientserverapi.jbi.management.admin.AdminService;
import org.ow2.petals.clientserverapi.jbi.management.deployment.DeploymentService;
import org.ow2.petals.clientserverapi.jbi.management.installation.InstallationService;
import org.ow2.petals.clientserverapi.system.logging.LoggingService;
import org.ow2.petals.clientserverapi.topology.TopologyService;
import org.ow2.petals.jmx.api.api.AdminServiceClient;
import org.ow2.petals.jmx.api.api.DeploymentServiceClient;
import org.ow2.petals.jmx.api.api.EndpointDirectoryClient;
import org.ow2.petals.jmx.api.api.InstallationServiceClient;
import org.ow2.petals.jmx.api.api.JMXClient;
import org.ow2.petals.jmx.api.api.LoggerServiceClient;
import org.ow2.petals.jmx.api.api.PetalsAdminServiceClient;
import org.ow2.petals.jmx.api.api.TopologyServiceClient;
import org.ow2.petals.jmx.api.impl.mbean.AdminServiceMBean;
import org.ow2.petals.jmx.api.impl.mbean.DeploymentServiceMBean;
import org.ow2.petals.jmx.api.impl.mbean.EndpointDirectoryService;
import org.ow2.petals.jmx.api.impl.mbean.EndpointDirectoryServiceMBean;
import org.ow2.petals.jmx.api.impl.mbean.InstallationServiceMBean;
import org.ow2.petals.jmx.api.impl.mbean.LoggingServiceMBean;
import org.ow2.petals.jmx.api.impl.mbean.PetalsAdminServiceMBean;
import org.ow2.petals.jmx.api.impl.mbean.TopologyServiceMBean;

/**
 * The EmbeddedJmxServerConnector Rule allows creation of an embedded JMX server connector to use Petals ESB MBeans that
 * are guaranteed to be deleted when the test method finishes (whether it passes or fails):
 * 
 * <pre>
 * public static class HasTempFolder {
 *  &#064;Rule
 *  public EmbeddedJmxServerConnector jmxSrvCon = new EmbeddedJmxServerConnector(port, credentials);
 * 
 *  &#064;Test
 *  public void test...() throws IOException {
 *      jmxSrvCon.setTopologyService(...);
 *      jmxSrvCon.setPetalsAdminService(...);
 *      ...
 *  }
 * }
 * </pre>
 */
public class EmbeddedJmxServerConnector extends ExternalResource {

    /**
     * The {@link JMXConnectorServer} listen on the localhost
     */
    private final static String JMX_HOST = "localhost";

    /**
     * The listening port of the {@link JMXConnectorServer}
     */
    private final int jmxPort;

    /**
     * Credentials of the {@link JMXConnectorServer}
     */
    private final String[][] credentials;

    /**
     * Temporary folder required to store password and access files needed to the {@link JMXConnectorServer}
     */
    private final TemporaryFolder tempFolder = new TemporaryFolder();

    /**
     * The JMX server of the current Java platform
     */
    private final MBeanServer mbeanServer = ManagementFactory.getPlatformMBeanServer();

    /**
     * The JMX connector server
     */
    private JMXConnectorServer jmxConnectorServer;

    /**
     * List of the registered MBeans
     */
    private List<ObjectName> registeredMBeans = new ArrayList<ObjectName>();

    /**
     * Name of the MBean 'PetalsAdminService'
     */
    private static ObjectName PETALSADMIN_MBEAN_NAME;

    /**
     * Name of the MBean 'TopologyService'
     */
    private static ObjectName TOPOLOGY_MBEAN_NAME;

    /**
     * Name of the MBean 'RegistryService'
     */
    private static ObjectName REGISTRY_MBEAN_NAME;

    /**
     * Name of the MBean 'LoggingService'
     */
    private static ObjectName LOGGING_MBEAN_NAME;

    /**
     * Name of the MBean 'InstallationService'
     */
    private static ObjectName INSTALLATION_MBEAN_NAME;

    /**
     * Name of the MBean 'DeploymentService'
     */
    private static ObjectName DEPLOYMENT_MBEAN_NAME;

    /**
     * Name of the MBean 'AdminService'
     */
    private static ObjectName ADMIN_MBEAN_NAME;

    /**
     * Name of the MBean 'LocalTransporterMonitoringService'
     */
    private static ObjectName LOCAL_TRANSPORTER_MONITORING_MBEAN_NAME;

    /**
     * Name of the MBean 'TcpTransporterMonitoringService'
     */
    private static ObjectName TCP_TRANSPORTER_MONITORING_MBEAN_NAME;

    static {
        try {
            PETALSADMIN_MBEAN_NAME = new ObjectName("Petals:name=" + PetalsAdminServiceClient.PETALS_ADMIN_MBEAN_NAME
                    + ",type=service");
            TOPOLOGY_MBEAN_NAME = new ObjectName("Petals:name=" + TopologyServiceClient.TOPOLOGY_SERVICE_MBEAN_NAME
                    + ",type=service");
            REGISTRY_MBEAN_NAME = new ObjectName("Petals:name=" + EndpointDirectoryClient.ENDPOINT_DIRECTORY_MBEAN_NAME
                    + ",type=service");
            LOGGING_MBEAN_NAME = new ObjectName("Petals:name=" + LoggerServiceClient.LOGGER_SERVICE_MBEAN_NAME
                    + ",type=service");
            INSTALLATION_MBEAN_NAME = new ObjectName("Petals:name="
                    + InstallationServiceClient.INSTALLATION_SERVICE_MBEAN_NAME + ",type=service");
            DEPLOYMENT_MBEAN_NAME = new ObjectName("Petals:name="
                    + DeploymentServiceClient.DEPLOYMENT_SERVICE_MBEAN_NAME + ",type=service");
            ADMIN_MBEAN_NAME = new ObjectName("Petals:name=" + AdminServiceClient.ADMIN_SERVICE_MBEAN_NAME
                    + ",type=service");
            LOCAL_TRANSPORTER_MONITORING_MBEAN_NAME = new ObjectName(
                    "Petals:name=" + "LocalTransporterMonitoring" + ",type=service");
            TCP_TRANSPORTER_MONITORING_MBEAN_NAME = new ObjectName(
                    "Petals:name=" + "TcpTransporterMonitoring" + ",type=service");
        } catch (final Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Creates an EmbeddedJmxServerConnector.
     * 
     * @param jmxPort
     *            The listening port of the {@link JMXConnectorServer}
     * @param credentials
     *            Credentials to secured the {@link JMXConnectorServer}
     */
    public EmbeddedJmxServerConnector(final int jmxPort, final String[][] credentials) {
        this.jmxPort = jmxPort;
        this.credentials = credentials;
    }

    /**
     * Creates an EmbeddedJmxServerConnector listening on the default JMX port ({@link JMXClient.DEFAULT_PORT}.
     * 
     * @param credentials
     *            Credentials to secured the {@link JMXConnectorServer}
     */
    public EmbeddedJmxServerConnector(final String[][] credentials) {
        this(JMXClient.DEFAULT_PORT, credentials);
    }

    /**
     * Creates an EmbeddedJmxServerConnector listening on the default JMX port ({@link JMXClient.DEFAULT_PORT} and using
     * the default credentials (petals/petals).
     */
    public EmbeddedJmxServerConnector() {
        this(JMXClient.DEFAULT_PORT, new String[][] { { "petals", "petals" } });
    }

    /**
     * Creates an EmbeddedJmxServerConnector using the default credentials (petals/petals).
     * 
     * @param jmxPort
     *            The listening port of the {@link JMXConnectorServer}
     */
    public EmbeddedJmxServerConnector(final int jmxPort) {
        this(jmxPort, new String[][] { { "petals", "petals" } });
    }

    @Override
    protected void before() throws Throwable {
        this.startJmxServerConnector();
    }

    @Override
    protected void after() {
        try {
            this.stopJmxServerConnector();
        } catch (final Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Create and start a {@link JMXConnectorServer}
     */
    private void startJmxServerConnector() throws Exception {

        final Registry registry = LocateRegistry.getRegistry(this.jmxPort);
        try {
            registry.list();
        } catch (Exception e) {
            LocateRegistry.createRegistry(this.jmxPort);
        }

        this.tempFolder.create();

        final HashMap<String, Object> environment;
        if (this.credentials != null && this.credentials.length > 0) {
            final File passwordFile = this.tempFolder.newFile("password.properties");
            final File accessFile = this.tempFolder.newFile("access.properties");

            final Properties pwdProps = new Properties();
            for (final String[] credential : this.credentials) {
                pwdProps.setProperty(credential[0], credential[1] == null ? "" : credential[1]);
            }
            final FileOutputStream fosPwd = new FileOutputStream(passwordFile);
            try {
                pwdProps.store(fosPwd, "Credentials file");
            } finally {
                fosPwd.close();
            }

            final Properties accessProps = new Properties();
            for (final String[] credential : this.credentials) {
                accessProps.setProperty(credential[0], "readwrite");
            }
            final FileOutputStream fosAccess = new FileOutputStream(accessFile);
            try {
                accessProps.store(fosAccess, "Access file");
            } finally {
                fosAccess.close();
            }

            environment = new HashMap<String, Object>();
            environment.put("jmx.remote.x.password.file", passwordFile.getAbsolutePath());
            environment.put("jmx.remote.x.access.file", accessFile.getAbsolutePath());
        } else {
            environment = null;
        }

        final JMXServiceURL address = new JMXServiceURL("service:jmx:rmi:///jndi/rmi://" + JMX_HOST + ":" + this.jmxPort
                + "/" + JMXClient.PETALS_JMX_RMI_STUB_NAME);
        this.jmxConnectorServer = JMXConnectorServerFactory.newJMXConnectorServer(address, environment,
                this.mbeanServer);
        this.jmxConnectorServer.start();

        this.registeredMBeans.clear();

    }

    /**
     * Stop and destroy the {@link JMXConnectorServer}
     */
    private void stopJmxServerConnector() throws Exception {
        for (final ObjectName registeredMbean : this.registeredMBeans) {
            this.mbeanServer.unregisterMBean(registeredMbean);
        }
        this.jmxConnectorServer.stop();
        this.tempFolder.delete();
    }

    public int getJmxPort() {
        return this.jmxPort;
    }

    /**
     * Register a MBean implementing {@link TopologyService}. If a MBean is already registered, it is unregistered
     * before registration of the new one.
     * 
     * @param topologyService
     *            The MBean implementing {@link TopologyService} to register.
     * @throws NotCompliantMBeanException
     * @throws RuntimeOperationsException
     * @throws InstanceNotFoundException
     * @throws MBeanException
     * @throws InvalidTargetObjectTypeException
     * @throws InstanceAlreadyExistsException
     *             The MBean was already registered, the unregistration succeeded but the new registration detects that
     *             the MBean exists always.
     */
    public EmbeddedJmxServerConnector registerTopologyService(final TopologyService topologyService)
            throws InstanceAlreadyExistsException,
            NotCompliantMBeanException, InstanceNotFoundException, RuntimeOperationsException, MBeanException,
            InvalidTargetObjectTypeException {

        this.registerService(TOPOLOGY_MBEAN_NAME,
                this.createCustomMBeanModel(topologyService, TopologyServiceMBean.class));

        return this;
    }

    /**
     * Register a MBean implementing {@link PetalsAdminService}. If a MBean is already registered, it is unregistered
     * before registration of the new one.
     * 
     * @param petalsAdminService
     *            The MBean implementing {@link PetalsAdminService} to register.
     * @throws NotCompliantMBeanException
     * @throws RuntimeOperationsException
     * @throws InstanceNotFoundException
     * @throws MBeanException
     * @throws InvalidTargetObjectTypeException
     * @throws InstanceAlreadyExistsException
     *             The MBean was already registered, the unregistration succeeded but the new registration detects that
     *             the MBean exists always.
     */
    public EmbeddedJmxServerConnector registerPetalsAdminService(final PetalsAdminService petalsAdminService)
            throws InstanceAlreadyExistsException, NotCompliantMBeanException, InstanceNotFoundException,
            RuntimeOperationsException, MBeanException, InvalidTargetObjectTypeException {

        this.registerService(PETALSADMIN_MBEAN_NAME,
                this.createCustomMBeanModel(petalsAdminService, PetalsAdminServiceMBean.class));

        return this;
    }

    /**
     * Register a MBean implementing {@link EndpointDirectoryService}. If a MBean is already registered, it is unregistered
     * before registration of the new one.
     * 
     * @param registryService
     *            The MBean implementing {@link EndpointDirectoryService} to register.
     * @throws NotCompliantMBeanException
     * @throws RuntimeOperationsException
     * @throws InstanceNotFoundException
     * @throws MBeanException
     * @throws InvalidTargetObjectTypeException
     * @throws InstanceAlreadyExistsException
     *             The MBean was already registered, the unregistration succeeded but the new registration detects that
     *             the MBean exists always.
     */
    public EmbeddedJmxServerConnector registerRegistryService(final EndpointDirectoryService registryService)
            throws InstanceAlreadyExistsException,
            NotCompliantMBeanException, RuntimeOperationsException, MBeanException, InstanceNotFoundException,
            InvalidTargetObjectTypeException {

        this.registerService(REGISTRY_MBEAN_NAME,
                this.createCustomMBeanModel(registryService, EndpointDirectoryServiceMBean.class));

        return this;
    }

    /**
     * Register a MBean implementing {@link LoggingService}. If a MBean is already registered, it is unregistered before
     * registration of the new one.
     * 
     * @param loggingService
     *            The MBean implementing {@link LoggingService} to register.
     * @throws NotCompliantMBeanException
     * @throws RuntimeOperationsException
     * @throws InstanceNotFoundException
     * @throws MBeanException
     * @throws InvalidTargetObjectTypeException
     * @throws InstanceAlreadyExistsException
     *             The MBean was already registered, the unregistration succeeded but the new registration detects that
     *             the MBean exists always.
     */
    public EmbeddedJmxServerConnector registerLoggerService(final LoggingService loggingService)
            throws InstanceAlreadyExistsException,
            NotCompliantMBeanException, RuntimeOperationsException, InstanceNotFoundException, MBeanException,
            InvalidTargetObjectTypeException {

        this.registerService(LOGGING_MBEAN_NAME, this.createCustomMBeanModel(loggingService, LoggingServiceMBean.class));

        return this;
    }

    /**
     * Register a MBean implementing {@link InstallationService}. If a MBean is already registered, it is unregistered
     * before registration of the new one.
     * 
     * @param installationService
     *            The MBean implementing {@link InstallationService} to register.
     * @throws NotCompliantMBeanException
     * @throws RuntimeOperationsException
     * @throws InstanceNotFoundException
     * @throws MBeanException
     * @throws InvalidTargetObjectTypeException
     * @throws InstanceAlreadyExistsException
     *             The MBean was already registered, the unregistration succeeded but the new registration detects that
     *             the MBean exists always.
     */
    public EmbeddedJmxServerConnector registerInstallationService(final InstallationService installationService)
            throws NotCompliantMBeanException, RuntimeOperationsException, InstanceNotFoundException, MBeanException,
            InvalidTargetObjectTypeException, InstanceAlreadyExistsException {

        this.registerService(INSTALLATION_MBEAN_NAME,
                this.createCustomMBeanModel(installationService, InstallationServiceMBean.class));

        return this;
    }

    /**
     * Register a MBean implementing {@link DeploymentService}. If a MBean is already registered, it is unregistered
     * before registration of the new one.
     * 
     * @param deploymentService
     *            The MBean implementing {@link DeploymentService} to register.
     * @throws NotCompliantMBeanException
     * @throws RuntimeOperationsException
     * @throws InstanceNotFoundException
     * @throws MBeanException
     * @throws InvalidTargetObjectTypeException
     * @throws InstanceAlreadyExistsException
     *             The MBean was already registered, the unregistration succeeded but the new registration detects that
     *             the MBean exists always.
     */
    public EmbeddedJmxServerConnector registerDeploymentService(final DeploymentService deploymentService)
            throws NotCompliantMBeanException,
            RuntimeOperationsException, InstanceNotFoundException, MBeanException, InvalidTargetObjectTypeException,
            InstanceAlreadyExistsException {

        this.registerService(DEPLOYMENT_MBEAN_NAME,
                this.createCustomMBeanModel(deploymentService, DeploymentServiceMBean.class));

        return this;
    }

    /**
     * Register a MBean implementing {@link AdminServiceMBean}. If a MBean is already registered, it is unregistered
     * before registration of the new one.
     * 
     * @param adminService
     *            The MBean implementing {@link AdminService} to register.
     * @throws NotCompliantMBeanException
     * @throws RuntimeOperationsException
     * @throws InstanceNotFoundException
     * @throws MBeanException
     * @throws InvalidTargetObjectTypeException
     * @throws InstanceAlreadyExistsException
     *             The MBean was already registered, the unregistration succeeded but the new registration detects that
     *             the MBean exists always.
     */
    public EmbeddedJmxServerConnector registerAdminService(final AdminService adminService)
            throws NotCompliantMBeanException,
            RuntimeOperationsException, InstanceNotFoundException, MBeanException, InvalidTargetObjectTypeException,
            InstanceAlreadyExistsException {

        this.registerService(ADMIN_MBEAN_NAME, this.createCustomMBeanModel(adminService, AdminServiceMBean.class));

        return this;
    }

    /**
     * Register a MBean implementing {@link DynamicMBean}. If a MBean is already registered, it is unregistered before
     * registration of the new one.
     * 
     * @param componentService
     *            The MBean implementing {@link InstallerMBean} to register.
     * @param componentId
     *            JBI identifier of the associated component
     * @throws NotCompliantMBeanException
     * @throws RuntimeOperationsException
     * @throws InstanceNotFoundException
     * @throws MBeanException
     * @throws InvalidTargetObjectTypeException
     * @throws InstanceAlreadyExistsException
     *             The MBean was already registered, the unregistration succeeded but the new registration detects that
     *             the MBean exists always.
     * @throws MalformedObjectNameException
     */
    public EmbeddedJmxServerConnector registerInstallerComponentService(final InstallerMBean installerComponentService,
            final String componentId) throws InstanceAlreadyExistsException, NotCompliantMBeanException,
            RuntimeOperationsException, InstanceNotFoundException, MBeanException, InvalidTargetObjectTypeException,
            MalformedObjectNameException {

        this.registerService(new ObjectName("Petals:name=" + componentId + ",type=installer"),
                this.createCustomMBeanModel(installerComponentService, InstallerMBean.class));

        return this;
    }

    /**
     * Register a MBean implementing {@link DynamicMBean}. If a MBean is already registered, it is unregistered before
     * registration of the new one.
     * 
     * @param configurationInstallerComponentService
     *            The MBean implementing {@link DynamicMBean} to register.
     * @param componentId
     *            JBI identifier of the associated component
     * @throws NotCompliantMBeanException
     * @throws RuntimeOperationsException
     * @throws InstanceNotFoundException
     * @throws MBeanException
     * @throws InvalidTargetObjectTypeException
     * @throws InstanceAlreadyExistsException
     *             The MBean was already registered, the unregistration succeeded but the new registration detects that
     *             the MBean exists always.
     * @throws MalformedObjectNameException
     */
    public EmbeddedJmxServerConnector registerConfigurationInstallerComponentService(
            final DynamicMBean configurationInstallerComponentService, final String componentId)
            throws InstanceAlreadyExistsException, NotCompliantMBeanException, RuntimeOperationsException,
            InstanceNotFoundException, MBeanException, InvalidTargetObjectTypeException, MalformedObjectNameException {

        this.registerService(new ObjectName("org.ow2.petals:type=custom,name=bootstrap_" + componentId),
                configurationInstallerComponentService);

        return this;
    }

    /**
     * Register a MBean implementing {@link ComponentLifeCycleMBean}. If a MBean is already registered, it is
     * unregistered before registration of the new one.
     * 
     * @param componentLifeCycleService
     *            The MBean implementing {@link ComponentLifeCycleMBean} to register.
     * @param componentId
     *            JBI identifier of the associated component
     * @throws NotCompliantMBeanException
     * @throws RuntimeOperationsException
     * @throws InstanceNotFoundException
     * @throws MBeanException
     * @throws InvalidTargetObjectTypeException
     * @throws InstanceAlreadyExistsException
     *             The MBean was already registered, the unregistration succeeded but the new registration detects that
     *             the MBean exists always.
     * @throws MalformedObjectNameException
     */
    public EmbeddedJmxServerConnector registerComponentLifeCycleService(
            final ComponentLifeCycleMBean componentLifeCycleService,
            final String componentId) throws InstanceAlreadyExistsException, NotCompliantMBeanException,
            RuntimeOperationsException, InstanceNotFoundException, MBeanException, InvalidTargetObjectTypeException,
            MalformedObjectNameException {

        this.registerService(new ObjectName("org.ow2.petals:type=binding,name=" + componentId),
                this.createCustomMBeanModel(componentLifeCycleService, ComponentLifeCycleMBean.class));

        return this;
    }

    /**
     * Register a MBean implementing {@link DynamicMBean}. If a MBean is already registered, it is unregistered before
     * registration of the new one.
     * 
     * @param runtimeConfigurationComponentService
     *            The MBean implementing {@link DynamicMBean} to register.
     * @param componentId
     *            JBI identifier of the associated component
     * @throws NotCompliantMBeanException
     * @throws RuntimeOperationsException
     * @throws InstanceNotFoundException
     * @throws MBeanException
     * @throws InvalidTargetObjectTypeException
     * @throws InstanceAlreadyExistsException
     *             The MBean was already registered, the unregistration succeeded but the new registration detects that
     *             the MBean exists always.
     * @throws MalformedObjectNameException
     */
    public EmbeddedJmxServerConnector registerRuntimeConfigurationComponentService(
            final DynamicMBean runtimeConfigurationComponentService,
            final String componentId) throws InstanceAlreadyExistsException, NotCompliantMBeanException,
            RuntimeOperationsException, InstanceNotFoundException, MBeanException, InvalidTargetObjectTypeException,
            MalformedObjectNameException {

        this.registerService(new ObjectName("org.ow2.petals:type=custom,name=runtime_configuration_" + componentId),
                runtimeConfigurationComponentService);

        return this;
    }

    /**
     * Register a MBean implementing
     * {@link org.ow2.petals.component.framework.clientserver.api.monitoring.MonitoringService}. If a MBean is already
     * registered, it is unregistered before registration of the new one.
     * 
     * @param componentMonitoringService
     *            The MBean implementing
     *            {@link org.ow2.petals.component.framework.clientserver.api.monitoring.MonitoringService} to register.
     * @param componentId
     *            JBI identifier of the associated component
     * @throws NotCompliantMBeanException
     * @throws RuntimeOperationsException
     * @throws InstanceNotFoundException
     * @throws MBeanException
     * @throws InvalidTargetObjectTypeException
     * @throws InstanceAlreadyExistsException
     *             The MBean was already registered, the unregistration succeeded but the new registration detects that
     *             the MBean exists always.
     * @throws MalformedObjectNameException
     */
    public EmbeddedJmxServerConnector registerComponentMonitoringService(
            final org.ow2.petals.component.framework.clientserver.api.monitoring.MonitoringService componentMonitoringService,
            final String componentId) throws InstanceAlreadyExistsException, NotCompliantMBeanException,
            RuntimeOperationsException, InstanceNotFoundException, MBeanException, InvalidTargetObjectTypeException,
            MalformedObjectNameException {

        this.registerService(new ObjectName("org.ow2.petals:type=custom,name=monitoring_" + componentId),
                componentMonitoringService);

        return this;
    }

    /**
     * Register a MBean implementing {@link org.ow2.petals.binding.soap.clientserver.api.monitoring.MonitoringService}.
     * If a MBean is already registered, it is unregistered before registration of the new one.
     * 
     * @param petalsBcSoapMonitoringService
     *            The MBean implementing
     *            {@link org.ow2.petals.binding.soap.clientserver.api.monitoring.MonitoringService} to register.
     * @param componentId
     *            JBI identifier of the associated component
     * @throws NotCompliantMBeanException
     * @throws RuntimeOperationsException
     * @throws InstanceNotFoundException
     * @throws MBeanException
     * @throws InvalidTargetObjectTypeException
     * @throws InstanceAlreadyExistsException
     *             The MBean was already registered, the unregistration succeeded but the new registration detects that
     *             the MBean exists always.
     * @throws MalformedObjectNameException
     */
    public EmbeddedJmxServerConnector registerPetalsBcSoapMonitoringService(
            final org.ow2.petals.binding.soap.clientserver.api.monitoring.MonitoringService petalsBcSoapMonitoringService,
            final String componentId) throws InstanceAlreadyExistsException, NotCompliantMBeanException,
            RuntimeOperationsException, InstanceNotFoundException, MBeanException, InvalidTargetObjectTypeException,
            MalformedObjectNameException {

        this.registerService(new ObjectName("org.ow2.petals:type=custom,name=monitoring_" + componentId),
                petalsBcSoapMonitoringService);

        return this;
    }

    /**
     * Register a MBean implementing
     * {@link org.ow2.petals.clientserverapi.transport.monitoring.LocalTransporterMonitoringService}. If a MBean is
     * already registered, it is unregistered before registration of the new one.
     * 
     * @param localTransporterMonitoringService
     *            The MBean implementing
     *            {@link org.ow2.petals.clientserverapi.transport.monitoring.LocalTransporterMonitoringService} to
     *            register.
     * @throws NotCompliantMBeanException
     * @throws RuntimeOperationsException
     * @throws InstanceNotFoundException
     * @throws MBeanException
     * @throws InvalidTargetObjectTypeException
     * @throws InstanceAlreadyExistsException
     *             The MBean was already registered, the unregistration succeeded but the new registration detects that
     *             the MBean exists always.
     * @throws MalformedObjectNameException
     */
    public EmbeddedJmxServerConnector registerLocalTransporterMonitoringService(
            final org.ow2.petals.clientserverapi.transport.monitoring.LocalTransporterMonitoringService localTransporterMonitoringService)
                    throws InstanceAlreadyExistsException, NotCompliantMBeanException, RuntimeOperationsException,
                    InstanceNotFoundException, MBeanException, InvalidTargetObjectTypeException,
                    MalformedObjectNameException {

        this.registerService(LOCAL_TRANSPORTER_MONITORING_MBEAN_NAME, localTransporterMonitoringService);

        return this;
    }

    /**
     * Register a MBean implementing
     * {@link org.ow2.petals.clientserverapi.transport.monitoring.TcpTransporterMonitoringService}. If a MBean is
     * already registered, it is unregistered before registration of the new one.
     * 
     * @param tcpTransporterMonitoringService
     *            The MBean implementing
     *            {@link org.ow2.petals.clientserverapi.transport.monitoring.TcpTransporterMonitoringService} to
     *            register.
     * @throws NotCompliantMBeanException
     * @throws RuntimeOperationsException
     * @throws InstanceNotFoundException
     * @throws MBeanException
     * @throws InvalidTargetObjectTypeException
     * @throws InstanceAlreadyExistsException
     *             The MBean was already registered, the unregistration succeeded but the new registration detects that
     *             the MBean exists always.
     * @throws MalformedObjectNameException
     */
    public EmbeddedJmxServerConnector registerTcpTransporterMonitoringService(
            final org.ow2.petals.clientserverapi.transport.monitoring.TcpTransporterMonitoringService tcpTransporterMonitoringService)
                    throws InstanceAlreadyExistsException, NotCompliantMBeanException, RuntimeOperationsException,
                    InstanceNotFoundException, MBeanException, InvalidTargetObjectTypeException,
                    MalformedObjectNameException {

        this.registerService(TCP_TRANSPORTER_MONITORING_MBEAN_NAME, tcpTransporterMonitoringService);

        return this;
    }

    private RequiredModelMBean createCustomMBeanModel(final Object service, final Class<?> mbeanClass)
            throws RuntimeOperationsException, MBeanException, InstanceNotFoundException,
            InvalidTargetObjectTypeException {

        // The RegistryMBean has a custom MBean model
        final Vector<ModelMBeanOperationInfo> operators = new Vector<ModelMBeanOperationInfo>();
        for (final Method method : mbeanClass.getMethods()) {
            final Descriptor desc = new DescriptorSupport();
            desc.setField("name", method.getName());
            desc.setField("descriptorType", "operation");
            desc.setField("role", "operation");
            final ModelMBeanOperationInfo mmoi = new ModelMBeanOperationInfo(method.getName(), method, desc);
            operators.add(mmoi);
        }
        final ModelMBeanOperationInfo[] operatorArray = new ModelMBeanOperationInfo[operators.size()];
        operators.copyInto(operatorArray);
        final Descriptor desc = new DescriptorSupport();
        desc.setField("name", service.getClass().getName());
        desc.setField("descriptorType", "MBean");
        final ModelMBeanInfo mminfo = new ModelMBeanInfoSupport(service.getClass().getName(), service.getClass()
                .getName(), null, new ModelMBeanConstructorInfo[0], operatorArray, new ModelMBeanNotificationInfo[0],
                desc);
        final RequiredModelMBean model = new RequiredModelMBean(mminfo);
        model.setManagedResource(service, "ObjectReference");

        return model;
    }

    /**
     * Register a MBean in the JMX server
     * 
     * @param objName
     *            The MBean name.
     * @param mbean
     *            The MBean to register.
     * @throws NotCompliantMBeanException
     * @throws RuntimeOperationsException
     * @throws InstanceNotFoundException
     * @throws MBeanException
     * @throws InvalidTargetObjectTypeException
     * @throws InstanceAlreadyExistsException
     *             The MBean was already registered, the unregistration succeeded but the new registration detects that
     *             the MBean exists always.
     */
    private void registerService(final ObjectName objName, final Object mbean) throws MBeanRegistrationException,
            InstanceNotFoundException, NotCompliantMBeanException, InstanceAlreadyExistsException {
        try {
            this.mbeanServer.registerMBean(mbean, objName);
            this.registeredMBeans.add(objName);
        } catch (final InstanceAlreadyExistsException e) {
            this.mbeanServer.unregisterMBean(objName);
            this.mbeanServer.registerMBean(mbean, objName);
        }
    }

}
